package com.jeecms.system.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.ObjectUtil;
import com.google.common.collect.ImmutableList;
import com.jeecms.auth.domain.CoreRole;
import com.jeecms.auth.domain.CoreUser;
import com.jeecms.auth.domain.dto.CmsOrgAgent;
import com.jeecms.auth.domain.dto.CoreRoleAgent;
import com.jeecms.auth.service.CoreRoleService;
import com.jeecms.auth.service.CoreUserService;
import com.jeecms.auth.service.ISiteAuthService;
import com.jeecms.auth.util.AuthUtil;
import com.jeecms.channel.domain.Channel;
import com.jeecms.channel.service.ChannelService;
import com.jeecms.common.base.service.BaseServiceImpl;
import com.jeecms.common.base.service.ICacheService;
import com.jeecms.common.exception.GlobalException;
import com.jeecms.common.jpa.support.Wrappers;
import com.jeecms.common.page.Paginable;
import com.jeecms.common.page.PaginableRequest;
import com.jeecms.common.util.GsonUtil;
import com.jeecms.common.util.TreeUtil;
import com.jeecms.common.web.util.RequestUtils;
import com.jeecms.component.listener.ChannelListener;
import com.jeecms.component.listener.SiteListener;
import com.jeecms.system.constants.AuthConstant;
import com.jeecms.system.constants.CacheConstant;
import com.jeecms.system.dao.SysAuthDataDao;
import com.jeecms.system.domain.CmsOrg;
import com.jeecms.system.domain.CmsSite;
import com.jeecms.system.domain.SysAuthData;
import com.jeecms.system.domain.dto.OrgPermDto;
import com.jeecms.system.domain.dto.OrgPermUnitDto;
import com.jeecms.system.domain.dto.UserPermDto;
import com.jeecms.system.domain.dto.UserPermUnitDto;
import com.jeecms.system.domain.vo.*;
import com.jeecms.system.domain.vo.CmsDataPermVo.MiniDataUnit;
import com.jeecms.system.dto.*;
import com.jeecms.system.dto.bo.*;
import com.jeecms.system.service.CmsOrgService;
import com.jeecms.system.service.CmsSiteService;
import com.jeecms.system.service.SysAuthDataService;
import com.jeecms.system.vo.*;
import com.jeecms.util.SystemContextUtils;
import com.jeecms.utils.GsonUtils;
import com.jeecms.utils.StrUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.jeecms.auth.constants.AuthCommonConstants.*;
import static com.jeecms.system.constants.AuthConstant.*;
import static com.jeecms.system.constants.CacheConstant.SITE_TREE_KEY;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author tom
 * @date 2022-05-17
 * @copyright 杭州藏茗山科技有限公司 All rights reserved. Notice
 * 仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysAuthDataServiceImpl extends BaseServiceImpl<SysAuthData, SysAuthDataDao, Integer> implements SysAuthDataService, ChannelListener, SiteListener, InitializingBean {

    private static final Logger log = LoggerFactory.getLogger(SysAuthDataServiceImpl.class);
    Paginable maxPaginable = new PaginableRequest(0, Integer.MAX_VALUE);

    @Resource
    private ICacheService cacheService;
    @Resource
    private CmsOrgService orgService;
    @Resource
    private CoreRoleService roleService;
    @Resource
    private CoreUserService userService;
    @Resource
    private CmsSiteService siteService;
    @Resource
    private ChannelService channelService;
    @Autowired
    private ISiteAuthService authService;

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public List<SysAuthData> getList(List<Integer> orgIds, Integer roleId, Integer userId, Integer siteId, Integer dataId, Integer dataType) {
        return dao.getList(orgIds, roleId, userId, siteId, dataId, dataType);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orgSite(DataOrgDTO dto) throws GlobalException {
        Map<String, Integer> map = new HashMap<>(10);
        List<SysAuthData> dataList = new ArrayList<>();
        //得到权限数据
        List<SysAuthData> authDatas = dao.getList(Collections.singletonList(dto.getOrgId()), null, null, null, null, SITE_TYPE);
        CmsOrg org = orgService.findById(dto.getOrgId());
        //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
        Map<Integer, Integer> orgAuthMap = getDtoAuthMap(dto.getSites());
        if (org.getParent() != null) {
            Map<Integer, Integer> parentOrgAuthMap = getOrgAuthMap(org.getParent(), true, SITE_TYPE, null);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(orgAuthMap, parentOrgAuthMap);
            if (authEqOwnerOrg && !authDatas.isEmpty()) {
                super.physicalDeleteInBatch(authDatas);
            } else {
                if (authDatas.isEmpty()) {
                    //如果之前没配置权限，直接新增
                    save(dto.getSites(), dataList, dto.getOrgId(), AuthConstant.AuthEnum.ORG_TYPE);
                } else {
                    //将之前的dto转换bo
                    dealWithData(covertBo(dto.getSites()), authDatas, dataList, dto.getOrgId(), dto.getOrgId(), AuthConstant.AuthEnum.ORG_TYPE);
                }
                super.saveAll(dataList);
                //处理修改缓存
                for (SysAuthData authData : dataList) {
                    map.put(CacheConstant.getSiteAuthOrgKey(authData.getOrgId(), authData.getDataId()),
                            authData.getDataAuth());
                }
            }
            clearCacheForOrgSiteAuth(org.getId());
        }
        cacheService.multiSet(map);
    }

    /**
     * 递归新增 --站点权限
     **/
    void save(List<DataSiteBaseDTO> sites, List<SysAuthData> dataList, Integer id, AuthEnum type) {
        if (AuthEnum.ORG_TYPE.equals(type)) {
            if (sites != null && !sites.isEmpty()) {
                for (DataSiteBaseDTO site : sites) {
                    SysAuthData data = site.toOrgEntity(id);
                    if (data.getDataAuth() != 0) {
                        dataList.add(data);
                    }
                    save(site.getChildren(), dataList, id, type);
                }
            }
        } else if (AuthEnum.ROLE_TYPE.equals(type)) {
            if (sites != null && !sites.isEmpty()) {
                for (DataSiteBaseDTO site : sites) {
                    SysAuthData data = site.toRoleEntity(id);
                    if (data.getDataAuth() != 0) {
                        dataList.add(data);
                    }
                    save(site.getChildren(), dataList, id, type);
                }
            }
        } else if (AuthEnum.USER_TYPE.equals(type)) {
            if (sites != null && !sites.isEmpty()) {
                for (DataSiteBaseDTO site : sites) {
                    SysAuthData data = site.toUserEntity(id);
                    if (data.getDataAuth() != 0) {
                        dataList.add(data);
                    }
                    save(site.getChildren(), dataList, id, type);
                }
            }
        }
    }

    /**
     * 获取站点类权限dto的权限map
     **/
    private Map<Integer, Integer> getDtoAuthMap(List<DataSiteBaseDTO> sites) {
        Map<Integer, Integer> dtoAuthMap = new HashMap<>();
        if (sites != null && !sites.isEmpty()) {
            for (DataSiteBaseDTO site : sites) {
                site.putAuthToMap(dtoAuthMap);
            }
        }
        return dtoAuthMap;
    }

    private Map<Integer, Integer> getDtoChannelAuthMap(List<DataChannelBaseDTO> channels, Integer siteId) {
        Map<Integer, Integer> dtoAuthMap = new HashMap<>();
        if (channels != null && !channels.isEmpty()) {
            for (DataChannelBaseDTO dto : channels) {
                dto.putAuthToMap(dtoAuthMap, siteId);
            }
        }
        return dtoAuthMap;
    }

    private Map<Integer, Integer> getDtoContentAuthMap(List<DataContentBaseDTO> channels, Integer siteId) {
        Map<Integer, Integer> dtoAuthMap = new HashMap<>();
        if (channels != null && !channels.isEmpty()) {
            for (DataContentBaseDTO dto : channels) {
                dto.putAuthToMap(dtoAuthMap, siteId);
            }
        }
        return dtoAuthMap;
    }


    /**
     * 递归转换--站点权限
     **/
    private List<DataSiteBaseBO> covertBo(List<DataSiteBaseDTO> sites) {
        List<DataSiteBaseBO> bos = new ArrayList<>();
        covertBoList(sites, bos);
        return bos;
    }

    /**
     * 递归转换--站点权限
     **/
    private void covertBoList(List<DataSiteBaseDTO> sites, List<DataSiteBaseBO> dataList) {
        if (sites != null && !sites.isEmpty()) {
            for (DataSiteBaseDTO site : sites) {
                dataList.add(site.toBO());
                covertBoList(site.getChildren(), dataList);
            }
        }
    }

    /**
     * 递归新增 --内容权限
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveContent(List<DataContentBaseDTO> contents, List<SysAuthData> dataList, Integer id, Integer siteId, AuthEnum type) {
        if (AuthEnum.ORG_TYPE.equals(type)) {
            if (contents != null && !contents.isEmpty()) {
                for (DataContentBaseDTO content : contents) {
                    dataList.add(content.toOrgEntity(id, siteId));
                    saveContent(content.getChildren(), dataList, id, siteId, type);
                }
            }
        } else if (AuthEnum.ROLE_TYPE.equals(type)) {
            if (contents != null && !contents.isEmpty()) {
                for (DataContentBaseDTO content : contents) {
                    dataList.add(content.toRoleEntity(id, siteId));
                    saveContent(content.getChildren(), dataList, id, siteId, type);
                }
            }
        } else if (AuthEnum.USER_TYPE.equals(type)) {
            if (contents != null && !contents.isEmpty()) {
                for (DataContentBaseDTO content : contents) {
                    dataList.add(content.toUserEntity(id, siteId));
                    saveContent(content.getChildren(), dataList, id, siteId, type);
                }
            }
        }
    }

    /**
     * 递归转换 -- 内容权限
     **/
    private List<DataContentBaseBO> covertBoForContent(List<DataContentBaseDTO> contents) {
        List<DataContentBaseBO> bos = new ArrayList<>();
        covertBoListForContent(contents, bos);
        return bos;
    }

    /**
     * 递归转换 -- 内容权限
     **/
    private void covertBoListForContent(List<DataContentBaseDTO> contents, List<DataContentBaseBO> dataList) {
        if (contents != null && !contents.isEmpty()) {
            for (DataContentBaseDTO content : contents) {
                dataList.add(content.toBO());
                covertBoListForContent(content.getChildren(), dataList);
            }
        }
    }

    /**
     * 递归新增 --栏目权限
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveChannel(List<DataChannelBaseDTO> channels, List<SysAuthData> dataList, Integer id, Integer siteId, AuthEnum type) {
        //前端之前数据这里是null，这里null统一设置-siteId，为了后续统一处理
//        if (channels != null && !channels.isEmpty()) {
//            List<DataChannelBaseDTO> newDataAuthKey = channels.stream().filter(x -> x.getChannelId() == null).collect(Collectors.toList());
//            if(newDataAuthKey.size()>1){
//                newDataAuthKey = newDataAuthKey.subList(0,1);
//            }
//            for(DataChannelBaseDTO c:newDataAuthKey){
//                c.setChannelId(-siteId);
//            }
//            channels = channels.stream().filter(x -> x.getChannelId() != null).collect(Collectors.toList());
//            channels.addAll(newDataAuthKey);
//        }
        if (AuthEnum.ORG_TYPE.equals(type)) {
            if (channels != null && !channels.isEmpty()) {
                for (DataChannelBaseDTO channel : channels) {
                    dataList.add(channel.toOrgEntity(id, siteId));
                    saveChannel(channel.getChildren(), dataList, id, siteId, type);
                }
            }
        } else if (AuthEnum.ROLE_TYPE.equals(type)) {
            if (channels != null && !channels.isEmpty()) {
                for (DataChannelBaseDTO channel : channels) {
                    dataList.add(channel.toRoleEntity(id, siteId));
                    saveChannel(channel.getChildren(), dataList, id, siteId, type);
                }
            }
        } else if (AuthEnum.USER_TYPE.equals(type)) {
            if (channels != null && !channels.isEmpty()) {
                for (DataChannelBaseDTO channel : channels) {
                    dataList.add(channel.toUserEntity(id, siteId));
                    saveChannel(channel.getChildren(), dataList, id, siteId, type);
                }
            }
        }
    }

    /**
     * 递归转换 -- 栏目权限
     **/
    private List<DataChannelBaseBO> covertBoForChannel(List<DataChannelBaseDTO> channels) {
        List<DataChannelBaseBO> bos = new ArrayList<>();
        covertBoListForChannel(channels, bos);
        return bos;
    }

    /**
     * 递归转换 -- 栏目权限
     **/
    private void covertBoListForChannel(List<DataChannelBaseDTO> contents, List<DataChannelBaseBO> dataList) {
        if (contents != null && !contents.isEmpty()) {
            for (DataChannelBaseDTO channel : contents) {
                dataList.add(channel.toBO());
                covertBoListForChannel(channel.getChildren(), dataList);
            }
        }
    }

    /**
     * 统一处理新增修改数据
     *
     * @param sites    站点数据
     * @param list     权限数据
     * @param dataList 装载集合
     * @param orgId    组织ID
     * @param id       主键，根据AuthEnum对应组织，角色，用户
     * @param authEnum 根据AuthEnum对应组织，角色，用户
     */
    private void dealWithData(List<DataSiteBaseBO> sites, List<SysAuthData> list,
                              List<SysAuthData> dataList, Integer orgId,
                              Integer id, AuthEnum authEnum) {
        //如果存在，做权限对比，只修改有改动的权限
        //1.先得到传来的数据
        List<String> list1 = sites.stream().map(x -> x.getSiteId() + StrUtil.UNDERLINE + AuthConstant.getAuthInt(x.getDatas().values()))
                .collect(Collectors.toList());
        //2.得到数据库的数据
        List<String> list2 = list.stream().map(x -> x.getDataId() + StrUtil.UNDERLINE + x.getDataAuth())
                .collect(Collectors.toList());
        //3.对比，得到修改了的数据
        list1.removeAll(list2);
        for (String x : list1) {
            String[] split = x.split(StrUtil.UNDERLINE);
            if (AuthEnum.ORG_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndOrgIdAndDataType(Integer.valueOf(split[0]), id, SITE_TYPE);
                if (data != null) {
                    Integer oldAuth = data.getDataAuth();
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                    //处理缓存，如果传值小于缓存值，则需要处理
                    if (Integer.parseInt(split[1]) < oldAuth) {
                        dealWithCacheForSite(id, Integer.valueOf(split[0]));
                    }
                } else {
                    if (Integer.parseInt(split[1]) != 0) {
                        SysAuthData data1 = new SysAuthData();
                        data1.setOrgId(id);
                        data1.setDataId(Integer.valueOf(split[0]));
                        data1.setDataType(SITE_TYPE);
                        data1.setDataAuth(Integer.valueOf(split[1]));
                        dataList.add(data1);
                    }
                }
            } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndRoleIdAndDataType(Integer.valueOf(split[0]), id, SITE_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                } else {
                    if (Integer.parseInt(split[1]) != 0) {
                        SysAuthData data1 = new SysAuthData();
                        data1.setRoleId(id);
                        data1.setDataId(Integer.valueOf(split[0]));
                        data1.setDataType(SITE_TYPE);
                        data1.setDataAuth(Integer.valueOf(split[1]));
                        dataList.add(data1);
                    }
                }
            } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndUserIdAndDataType(Integer.valueOf(split[0]), id, SITE_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                } else {
                    if (Integer.parseInt(split[1]) != 0) {
                        SysAuthData data1 = new SysAuthData();
                        data1.setUserId(id);
                        data1.setDataId(Integer.valueOf(split[0]));
                        data1.setDataType(SITE_TYPE);
                        data1.setDataAuth(Integer.valueOf(split[1]));
                        dataList.add(data1);
                    }
                }
            }
        }
        //4.删除多余的数据
        deleteCacheAndEntityForSite(sites, list, orgId, id, authEnum);
    }

    /***
     * 处理缓存---站点类
     * @param id 组织ID
     * @param siteId 站点ID
     */
    public void dealWithCacheForSite(Integer id, Integer siteId) {
        //处理本组织下的用户缓存
        //删除组织下面的子组织下的角色，用户全部的缓存
        List<CmsOrg> orgList = orgService.list(Wrappers.<CmsOrg>lambdaQuery().cacheable().eq(CmsOrg::getParentId, id));
        List<Integer> ids = orgList.stream().map(CmsOrg::getId).collect(Collectors.toList());
        ids.add(id);
        dealWithCacheForOrgSite(ids, siteId);
    }

    /***
     * 处理缓存---站点类
     * @param orgIds 组织ID
     * @param siteId 站点ID
     */
    private void dealWithCacheForOrgSite(List<Integer> orgIds, Integer siteId) {
        for (Integer orgId : orgIds) {
            Set<String> keys = cacheService.keys(CacheConstant.getSiteAuthDelKey(orgId, siteId));
            Set<String> keyList1 = cacheService.keys(CacheConstant.getSiteAuthUserAllKey(orgId));
            cacheService.del(keys);
            cacheService.del(keyList1);
        }
    }

    /***
     * 处理缓存---站点类 用户缓存
     * @param userIds 组织ID
     * @param siteId 站点ID
     */
    @Override
    public void dealWithCacheForUserSite(List<Integer> userIds, Integer siteId) {
        for (Integer userId : userIds) {
            Integer orgId = userService.findById(userId).getOrgId();
            Set<String> keys = cacheService.keys(CacheConstant.getSiteAuthUserKey(orgId, userId, siteId));
            cacheService.del(keys);
        }
    }

    /***
     * 处理缓存---栏目类
     * @param orgIds 组织ID
     * @param siteId 站点ID
     */
    private void dealWithCacheForOrgChannel(List<Integer> orgIds, Integer siteId, Integer channelId) {
        for (Integer orgId : orgIds) {
            Set<String> keys = cacheService.keys(CacheConstant.getChannelAuthDelKey(orgId, siteId, channelId));
            Set<String> keyList1 = cacheService.keys(CacheConstant.getChannelAuthUserAllKey(orgId));
            cacheService.del(keys);
            cacheService.del(keyList1);
            Set<String> keyList2 = cacheService.keys(CacheConstant.getChannelAuthForOrgSiteDelKey(orgId));
            cacheService.del(keyList2);
        }
    }

    /***
     * 处理缓存---栏目类
     * @param roleIds 角色ID
     * @param siteId 站点ID
     */
    private void dealWithCacheForChannelRole(List<Integer> roleIds, Integer siteId, Integer channelId) {
        for (Integer roleId : roleIds) {
            CoreRole role = roleService.findById(roleId);
            if (role != null) {
                Set<String> keyList1 = cacheService.keys(CacheConstant.getChannelAuthRoleKey(role.getOrgid(), roleId, siteId, channelId));
                cacheService.del(keyList1);
                //一同删除角色下用户权限缓存，用户有了就直接返回了
                for (CoreUser u : role.getUsers()) {
                    Set<String> uks = cacheService.keys(CacheConstant.getChannelAuthForUserForDel(u.getOrgId(), u.getId()));
                    cacheService.del(uks);
                }
                Set<String> keyList2 = cacheService.keys(CacheConstant.getChannelAuthForRoleSiteDelKey(roleId));
                cacheService.del(keyList2);
            }
        }
    }

    /***
     * 处理缓存---栏目类
     */
    private void dealWithCacheForChannelOrg(List<Integer> orgIds, Integer siteId, Integer channelId) {
        for (Integer orgId : orgIds) {
            Set<String> keyList1 = cacheService.keys(CacheConstant.getChannelAuthOrgKey(orgId, siteId, channelId));
            cacheService.del(keyList1);
            Set<String> keyList2 = cacheService.keys(CacheConstant.getChannelAuthForOrgSiteDelKey(orgId));
            cacheService.del(keyList2);
        }
    }

    /***
     * 处理缓存---文档类
     * @param roleIds 角色ID
     * @param siteId 站点ID
     */
    private void dealWithCacheForRoleContent(List<Integer> roleIds, Integer siteId, Integer channelId) {
        for (Integer roleId : roleIds) {
            CoreRole role = roleService.findById(roleId);
            if (role != null) {
                Set<String> keyList1 = cacheService.keys(CacheConstant.getContentAuthRoleKey(role.getOrgid(), roleId, siteId, channelId));
                cacheService.del(keyList1);
            }
            Set<String> keyList2 = cacheService.keys(CacheConstant.getContentAuthForRoleSiteDelKey(roleId));
            cacheService.del(keyList2);
        }
    }

    /***
     * 处理缓存---栏目类 用户缓存
     * @param userIds 组织ID
     * @param siteId 站点ID
     */
    @Override
    public void dealWithCacheForChannelUser(List<Integer> userIds, Integer siteId, Integer channelId) {
        for (Integer userId : userIds) {
            Integer orgId = userService.findById(userId).getOrgId();
            Set<String> keys2 = cacheService.keys(CacheConstant.getChannelAuthForUserForDel(orgId, userId));
            Set<String> keys = cacheService.keys(CacheConstant.getChannelAuthUserKey(orgId, userId, siteId, channelId));
            cacheService.del(keys);
            cacheService.del(keys2);
        }
    }

    /***
     * 处理缓存---内容类 用户缓存
     * @param userIds 组织ID
     * @param siteId 站点ID
     */
    @Override
    public void dealWithCacheForUserContent(List<Integer> userIds, Integer siteId, Integer channelId) {
        for (Integer userId : userIds) {
            Integer orgId = userService.findById(userId).getOrgId();
            Set<String> keys = cacheService.keys(CacheConstant.getContentAuthUserKey(orgId, userId, siteId, channelId));
            cacheService.del(keys);
        }
    }

    @Override
    public void clearCacheForUserChannelAuth(Integer orgId, Integer userId) {
        Set<String> keys4 = cacheService.keys(CacheConstant.getChannelAuthForUserForDel(orgId, userId));
        cacheService.del(keys4);
        Set<String> keys = cacheService.keys(CacheConstant.getAllChannelAuthUserKey(orgId, userId));
        cacheService.del(keys);
        Set<String> keys2 = cacheService.keys(CacheConstant.getAllSiteChannelAuthForUser(orgId, userId));
        cacheService.del(keys2);
        Set<String> keys3 = cacheService.keys(CacheConstant.getAllSiteViewChannelAuthForUser(orgId, userId));
        cacheService.del(keys3);
    }

    @Override
    public void clearCacheForUserContentAuth(Integer orgId, Integer userId) {
        Set<String> keys2 = cacheService.keys(CacheConstant.getAllSiteContentAuthForUser(orgId, userId));
        cacheService.del(keys2);
        Set<String> keys3 = cacheService.keys(CacheConstant.getAllSiteContentAuthNoPermForUser(orgId, userId));
        cacheService.del(keys3);
        Set<String> keys4 = cacheService.keys(CacheConstant.getAllSiteContentAuthWithPermForUser(orgId, userId));
        cacheService.del(keys4);
        Set<String> keys5 = cacheService.keys(CacheConstant.getContentAuthForUser(orgId, userId));
        cacheService.del(keys5);
        Set<String> keys1 = cacheService.keys(CacheConstant.getAllContentAuthOrgKey(orgId));
        cacheService.del(keys1);
    }

    @Override
    public void clearCacheForUserSiteAuth(Integer orgId, Integer userId) {
        //用户站点类权限列表map
        Set<String> keys = cacheService.keys(CacheConstant.getSiteAuthForUser(orgId, userId));
        cacheService.del(keys);
        //某条站点数据的缓存
        Set<String> keys2 = cacheService.keys(CacheConstant.getSiteAuthUserKeyForDel(orgId, userId));
        cacheService.del(keys2);

    }

    @Override
    public void clearCacheForOrgChannelAuth(Integer orgId) {
        Set<String> keys = cacheService.keys(CacheConstant.getAllChannelAuthOrgKey(orgId));
        cacheService.del(keys);
        Set<String> keys2 = cacheService.keys(CacheConstant.getChannelAuthForOrgSiteDelKey(orgId));
        cacheService.del(keys2);
        Set<String> keys3 = cacheService.keys(CacheConstant.getAuthCountOrgKey(orgId, CHANNEL_TYPE));
        cacheService.del(keys3);
        Set<String> keys4 = cacheService.keys(CacheConstant.getAuthCountOrgKey(orgId, CONTENT_TYPE));
        cacheService.del(keys4);
    }

    @Override
    public void clearCacheForOrgContentAuth(Integer orgId) {
        Set<String> keys = cacheService.keys(CacheConstant.getAllContentAuthOrgKey(orgId));
        cacheService.del(keys);
        Set<String> keys2 = cacheService.keys(CacheConstant.getContentAuthForOrgSiteDelKey(orgId));
        cacheService.del(keys2);
    }

    @Override
    public void clearCacheForOrgSiteAuth(Integer orgId) {
        Set<String> keys = cacheService.keys(CacheConstant.getAllSiteAuthOrgKey(orgId));
        cacheService.del(keys);
    }

    @Override
    public void clearCacheForRoleChannelAuth(Integer roleId) {
        CoreRole role = roleService.findById(roleId);
        if (role != null) {
            Set<String> keys = cacheService.keys(CacheConstant.getAllChannelAuthRoleKey(role.getOrgid(), roleId));
            cacheService.del(keys);
            Set<String> keys2 = cacheService.keys(CacheConstant.getChannelAuthForRoleSiteDelKey(roleId));
            cacheService.del(keys2);
        }
    }

    @Override
    public void clearCacheForRoleContentAuth(Integer roleId) {
        CoreRole role = roleService.findById(roleId);
        if (role != null) {
            Set<String> keys = cacheService.keys(CacheConstant.getAllContentAuthRoleKey(role.getOrgid(), roleId));
            cacheService.del(keys);
            Set<String> keys2 = cacheService.keys(CacheConstant.getContentAuthForRoleSiteDelKey(roleId));
            cacheService.del(keys2);
        }
    }

    @Override
    public void clearCacheForRoleSiteAuth(Integer roleId) {
        CoreRole role = roleService.findById(roleId);
        if (role != null) {
            Set<String> keys = cacheService.keys(CacheConstant.getAllSiteAuthRoleKey(role.getOrgid(), roleId));
            cacheService.del(keys);
        }
    }

    @Override
    public void clearCacheForUserBo(String username) {
        cacheService.del(cacheService.keys(CacheConstant.getAdminUserBO(username)));
    }

    /***
     * 处理缓存---站点类 用户缓存
     * @param roleIds 角色ID
     * @param siteId 站点ID
     */
    private void dealWithCacheForRolesSite(List<Integer> roleIds, Integer siteId) {
        for (Integer roleId : roleIds) {
            Integer orgId = roleService.findById(roleId).getOrgid();
            Set<String> keys = cacheService.keys(CacheConstant.getSiteAuthRoleKey(orgId, roleId, siteId));
            cacheService.del(keys);
        }
    }


    /***
     * 处理缓存---内容类
     * @param orgId 组织ID
     * @param siteId 站点ID
     */
    public void dealWithCacheForOrgContent(Integer orgId, Integer siteId, Integer channelId) {
        //处理本组织下的用户缓存
        //删除组织下面的子组织下的角色，用户全部的缓存
        //TODO 删除组织下面的子组织下的角色，用户全部的缓存 这个是有问题的，这里
        List<CmsOrg> orgList = orgService.list(Wrappers.<CmsOrg>lambdaQuery().cacheable().eq(CmsOrg::getParentId, orgId));
        List<Integer> ids = orgList.stream().map(CmsOrg::getId).collect(Collectors.toList());
        ids.add(orgId);
        dealWithCacheForOrgContent(ids, siteId, channelId);
    }

    /***
     * 处理缓存---内容类
     */
    public void dealWithCacheForOrgContent(List<Integer> orgIds, Integer siteId, Integer channelId) {
        for (Integer id : orgIds) {
            Set<String> keys = cacheService.keys(CacheConstant.getContentAuthDelKey(id, siteId, channelId));
            Set<String> keyList1 = cacheService.keys(CacheConstant.getContentAuthUserAllKey(id));
            cacheService.del(keys);
            cacheService.del(keyList1);
            Set<String> keyList2 = cacheService.keys(CacheConstant.getContentAuthForOrgSiteDelKey(id));
            cacheService.del(keyList2);
        }
    }

    /***
     * 处理缓存---栏目类
     * @param orgId 组织ID
     * @param siteId 站点ID
     */
    public void dealWithCacheForOrgChannel(Integer orgId, Integer siteId, Integer channelId) {
        //处理本组织下的用户缓存
        //删除组织下面的子组织下的角色，用户全部的缓存
        List<CmsOrg> orgList = orgService.list(Wrappers.<CmsOrg>lambdaQuery().cacheable().eq(CmsOrg::getParentId, orgId));
        List<Integer> ids = orgList.stream().map(CmsOrg::getId).collect(Collectors.toList());
        ids.add(orgId);
        for (Integer id : ids) {
            Set<String> keys = cacheService.keys(CacheConstant.getChannelAuthDelKey(id, siteId, channelId));
            Set<String> keyList1 = cacheService.keys(CacheConstant.getChannelAuthUserAllKey(id));
            cacheService.del(keys);
            cacheService.del(keyList1);
        }
    }

    /**
     * 删除redis---站点类权限
     **/
    public void deleteCacheAndEntityForSite(List<DataSiteBaseBO> sites, List<SysAuthData> list, Integer orgId,
                                            Integer id, AuthEnum authEnum) {
        List<Integer> del = sites.stream().map(DataSiteBaseBO::getSiteId).collect(Collectors.toList());
        Map<Integer, Integer> del2 = list.stream().collect(Collectors.toMap(SysAuthData::getId, SysAuthData::getDataId, (key1, key2) -> key2));
        del2.values().removeAll(del);
        Set<String> set = new HashSet<>(del2.size());
        if (AuthEnum.ORG_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getSiteAuthOrgKey(id, value)));
            cacheService.del(set);
        } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getSiteAuthRoleKey(orgId, id, value)));
            cacheService.del(set);
        } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getSiteAuthUserKey(orgId, id, value)));
            cacheService.del(set);
        }
        physicalDeleteByIdInBatch(del2.keySet());
    }

    /**
     * 删除redis---内容类权限
     **/
    public void deleteCacheAndEntityForContent(List<DataContentBaseBO> contents, List<SysAuthData> list,
                                               Integer orgId, Integer id, Integer siteId, AuthEnum authEnum) {
        List<Integer> del = contents.stream().map(DataContentBaseBO::getChannelId).collect(Collectors.toList());
        Map<Integer, Integer> del2 = list.stream().collect(Collectors.toMap(SysAuthData::getId, SysAuthData::getDataId, (key1, key2) -> key2));
        del2.values().removeAll(del);
        Set<String> set = new HashSet<>(del2.size());
        if (AuthEnum.ORG_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getContentAuthOrgKey(id, siteId, value)));
            cacheService.del(set);
        } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getContentAuthRoleKey(orgId, id, siteId, value)));
            cacheService.del(set);
        } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getContentAuthUserKey(orgId, id, siteId, value)));
            cacheService.del(set);
        }
        physicalDeleteByIdInBatch(del2.keySet());
    }

    /**
     * 删除redis---栏目类权限
     *
     * @param list         传过来的
     * @param authDataList 原来的
     * @param orgId        组织id
     * @param id           组织id/角色id/用户id
     * @param siteId       站点id
     * @param authEnum     类型 {@link AuthEnum}
     */
    public void deleteCacheAndEntityForChannel(List<DataChannelBaseBO> list, List<SysAuthData> authDataList,
                                               Integer orgId, Integer id, Integer siteId, AuthEnum authEnum) {
        List<Integer> del = list.stream().map(DataChannelBaseBO::getChannelId).collect(Collectors.toList());
        Map<Integer, Integer> del2 = authDataList.stream().collect(Collectors.toMap(SysAuthData::getId, SysAuthData::getDataId, (key1, key2) -> key2));
        del2.values().removeAll(del);
        Set<String> set = new HashSet<>(del2.size());
        if (AuthEnum.ORG_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getChannelAuthOrgKey(id, siteId, value)));
            cacheService.del(set);
        } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getChannelAuthRoleKey(orgId, id, siteId, value)));
            cacheService.del(set);
        } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
            del2.values().forEach(value -> set.add(CacheConstant.getChannelAuthUserKey(orgId, id, siteId, value)));
            cacheService.del(set);
        }
        physicalDeleteByIdInBatch(del2.keySet());
    }


    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public CmsDataPermVo orgSiteDetails(Integer orgId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<SiteDataPermTree> siteDataPermTrees = CollUtil.newArrayList();
        CmsOrg org = orgService.findById(orgId);
        vo.setDataIds(siteDataPermTrees);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        //得到站点树
        String data = getSiteData();
        if (StringUtils.isBlank(data)) {
            return vo;
        }
        if (org != null) {
            //特殊处理顶级组织
            if (org.isTop()) {
                vo.setManagerAble(false);
                List<SiteDataPermTree> dataPermTrees = getOrgSitePermTree(org);
                vo.setDataIds(dataPermTrees);
                return vo;
            }
            vo.setManagerAble(org.getDeleteAble());
            List<SysAuthData> orgAuthDataList = dao.getList(Collections.singletonList(orgId), null, null, null, null, SITE_TYPE);
            //有独立的权限数据则是单独分配了
            if (!orgAuthDataList.isEmpty()) {
                vo.setAssigned(true);
            }
            List<SiteDataPermTree> dataPermTrees = getOrgSitePermTree(org);
            vo.setDataIds(dataPermTrees);
        }
        return vo;
    }

    private List<SiteDataPermTree> getOrgSitePermTree(CmsOrg org) {
        List<SiteDataPermTree> dataPermTrees = CollUtil.newArrayList();
        //得到站点树
        String data = getSiteData();
        if (StringUtils.isBlank(data)) {
            return dataPermTrees;
        }
        List<AdminSiteTreeVO> siteTreeVos = GsonUtils.gsonToList(data, AdminSiteTreeVO.class);
        if (org != null) {
            //特殊处理顶级组织
            if (org.isTop()) {
                dataPermTrees = topOrgAndAdminForSite(siteTreeVos, dataPermTrees);
                //处理增量站点
                dealNewSite(dataPermTrees, null, null, null, true);
                return dataPermTrees;
            }
            CmsOrg parentOrg = org.getParent();
            //取所有上层所有父节点权限
            Map<Integer, Integer> parentAuthMap = getOrgAuthMap(parentOrg, true, SITE_TYPE, null);
            //得到权限数据
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(org, true, SITE_TYPE, null);
            Map<Integer, Integer> userAuthMap = getSiteAuth(SystemContextUtils.getCurrentUsername());
            //用户有权限还得父层有权限才可分配，此处与运算，userAuthMap 主要用于 convert方法判断是否可分配
            resultWith(userAuthMap, parentAuthMap);
            convert(siteTreeVos, orgAuthMap, parentAuthMap, userAuthMap, dataPermTrees, parentOrg.isTop());
            //树排序
            dataPermTrees = SiteDataPermTree.sort(dataPermTrees);
            dealNewSite(dataPermTrees, orgAuthMap, parentAuthMap, userAuthMap, false);
            return dataPermTrees;
        }
        return dataPermTrees;
    }

    /**
     * 获取组织全站点的 权限，主要是栏目和内容类权限
     *
     * @param org
     * @param forChildOrg
     * @param dataType
     * @return
     */
    private Map<Integer, Integer> getOrgAuthMapAllSite(CmsOrg org, boolean forChildOrg, int dataType) {
        List<CmsSite> sites = siteService.findAll(false, true);
        Map<Integer, Integer> map = new HashMap<>();
        for (CmsSite s : sites) {
            map.putAll(getOrgAuthMap(org, forChildOrg, dataType, s.getId()));
        }
        return map;
    }

    /**
     * 获取组织权限

     * @param org         组织
     * @param forChildOrg 是否是为子组织取， false 如果是角色或者用户需要判断组织自己是否有配置，如果有配置则以组织自身为准
     * @param dataType    数据类型
     * @param siteId      站点ID 站点类用不上，栏目类和文档类用
     * @return
     */
    private Map<Integer, Integer> getOrgAuthMap(CmsOrg org, boolean forChildOrg, int dataType, Integer siteId) {
        Map<Integer, Integer> orgAuthMap = MapUtil.newHashMap();
        String cacheKey = null;
        if (CHANNEL_TYPE == dataType) {
            cacheKey = CacheConstant.getChannelAuthForOrgSite(org.getId(), siteId);
        }
        if (CONTENT_TYPE == dataType) {
            cacheKey = CacheConstant.getContentAuthForOrgSite(org.getId(), siteId);
        }
        //暂时考虑栏目和内容类
        if (StringUtils.isNotBlank(cacheKey) && cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            orgAuthMap = GsonUtil.gsonToMaps(o.toString());
            return orgAuthMap;
        }
        //默认没数据给没权限
        if (SITE_TYPE == dataType) {
            orgAuthMap.put(0, 0);
        } else {
            orgAuthMap.put(-siteId, 0);
        }
        int allSelectVal = SITE_TYPE == dataType ? SITE_ALL_SELECT : (CHANNEL_TYPE == dataType ? CHANNEL_ALL_SELECT : CONTENT_ALL_SELECT);
        if (org != null) {
            //特殊处理顶级组织
            if (org.isTop()) {
                //处理增量站点
                if (SITE_TYPE == dataType) {
                    orgAuthMap.putAll(getTopOrgSiteAuthMap());
                } else {
                    orgAuthMap.putAll(getTopOrgChannelAuthMap(siteId, dataType));
                }
                if (StringUtils.isNotBlank(cacheKey)) {
                    cacheService.set(cacheKey, GsonUtils.gsonString(orgAuthMap));
                }
                return orgAuthMap;
            }
            CmsOrg parentOrg = org.getParent();
            //得到权限数据
            List<SysAuthData> orgAuthDataList = dao.getList(Collections.singletonList(org.getId()), null, null, null, null, dataType);
            orgAuthMap = orgAuthDataList.stream().collect(Collectors.toMap(SysAuthData::getDataId, SysAuthData::getDataAuth, (key1, key2) -> key2));
            //取所有上层所有父节点权限
            List<SysAuthData> parentAuthDataList = dao.getList(parentOrg.getNodeIdList(), null, null, null, null, dataType);
            Map<Integer, Integer> parentAuthMap = parentAuthDataList.stream().collect(Collectors.toMap(SysAuthData::getDataId, SysAuthData::getDataAuth, (key1, key2) -> key2));
            //自己也没有配置则取上层，上层没有配置取顶层权限
            if (orgAuthMap.isEmpty()) {
                orgAuthMap.putAll(parentAuthMap);
                if (parentAuthMap.isEmpty()) {
                    if (SITE_TYPE == dataType) {
                        orgAuthMap.putAll(getTopOrgSiteAuthMap());
                    } else {
                        orgAuthMap.putAll(getTopOrgChannelAuthMap(siteId, dataType));
                    }
                }
            }
            //增量数据权限key，站点类0 栏目类-siteId
            int newDataKey = 0;
            if (CHANNEL_TYPE == dataType || CONTENT_TYPE == dataType) {
                newDataKey = -siteId;
            }
            //处理增量站点 自己没有配置则默认没有，再总和上级，上级有他就有，上级没有他就没有
            int a = orgAuthMap.get(newDataKey) != null ? orgAuthMap.get(newDataKey) : 0;
            //上级未配置   自己未配置则是有所有,自己有配置则已自己为准
            int b = parentAuthMap.get(newDataKey) != null ? parentAuthMap.get(newDataKey) : allSelectVal;
            //如果是角色或用户来取组织权限，所属组织上配置了权限，则以自身权限为准
            if (!forChildOrg && !orgAuthMap.isEmpty()) {
                b = 0;
            }
            //子组织如果自身配置了权限 且父层非顶层则以自身权限为准
            if (forChildOrg && !orgAuthMap.isEmpty()) {
                b = 0;
            }
            orgAuthMap.put(newDataKey, a | b);
        }
        if (StringUtils.isNotBlank(cacheKey)) {
            cacheService.set(cacheKey, GsonUtils.gsonString(orgAuthMap));
        }
        return orgAuthMap;
    }

    private Map<Integer, Integer> getTopOrgSiteAuthMap() {
        Map<Integer, Integer> orgAuthMap = new HashMap<>();
        //站点类
        //得到站点树
        String data = getSiteData();
        if (StringUtils.isBlank(data)) {
            return orgAuthMap;
        }
        List<AdminSiteTreeVO> siteTreeVos = GsonUtils.gsonToList(data, AdminSiteTreeVO.class);
        orgAuthMap = topOrgAndAdminForSite(siteTreeVos, orgAuthMap);
        //处理增量站点
        orgAuthMap.put(0, SITE_ALL_SELECT);
        return orgAuthMap;
    }

    private Map<Integer, Integer> getTopOrgChannelAuthMap(Integer siteId, Integer dataType) {
        Map<Integer, Integer> orgAuthMap = new HashMap<>();
        //栏目类和文档类
        String data = getChannelData(siteId);
        if (StringUtils.isBlank(data)) {
            return orgAuthMap;
        }
        List<AdminChannelTreeVO> channelTreeVOS = GsonUtils.gsonToList(data, AdminChannelTreeVO.class);
        orgAuthMap = topOrgAndAdminForChannel(channelTreeVOS, orgAuthMap, dataType);
        int allSelectVal = CHANNEL_TYPE == dataType ? CHANNEL_ALL_SELECT : CONTENT_ALL_SELECT;
        //增量栏目的key 采用 站点id负数 这样不冲突
        orgAuthMap.put(-siteId, allSelectVal);
        return orgAuthMap;
    }

    private Map<Integer, Integer> getRoleAuthMapAllSite(CoreRole role, List<UserAuthBO> userAuthBOS, int dataType) {
        List<CmsSite> sites = siteService.findAll(false, true);
        Map<Integer, Integer> map = new HashMap<>();
        for (CmsSite s : sites) {
            map.putAll(getRoleAuthMap(role, userAuthBOS, dataType, s.getId()));
        }
        return map;
    }

    private Map<Integer, Integer> getRoleAuthMap(CoreRole role, List<UserAuthBO> userAuthBOS, int dataType, Integer siteId) {
        String cacheKey = null;
        if (CHANNEL_TYPE == dataType) {
            cacheKey = CacheConstant.getChannelAuthForRoleSite(role.getId(), siteId);
        }
        if (CONTENT_TYPE == dataType) {
            cacheKey = CacheConstant.getContentAuthForRoleSite(role.getId(), siteId);
        }
        //暂时考虑栏目和内容类
        if (StringUtils.isNotBlank(cacheKey) && cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            return GsonUtil.gsonToMaps(o.toString());
        }
        //得到角色的勾选权限
        List<SysAuthData> roleAuth = getList(null, role.getId(), null, null, null, dataType);
        //角色单独配置了以自身为准
        if (!roleAuth.isEmpty()) {
            Map<Integer, Integer> roleAuthMap = roleAuth.stream().collect(Collectors.toMap(SysAuthData::getDataId, SysAuthData::getDataAuth, (key1, key2) -> key2));
            //如果角色所属组织是顶层，则无需过滤计算
            if (role.getOrg().isTop()) {
                for (Map.Entry<Integer, Integer> ent : roleAuthMap.entrySet()) {
                    userAuthBOS.add(new UserAuthBO(ent.getKey(), ent.getValue()));
                }
            } else {
                //得到角色所属组织的勾选权限
                //做与运算
                Map<Integer, Integer> orgAuthMap = getOrgAuthMap(role.getOrg(), false, dataType, siteId);
                resultWith(roleAuthMap, orgAuthMap, userAuthBOS);
            }
        } else {
            //角色未配置权限的情况下继承所属组织
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(role.getOrg(), false, dataType, siteId);
            resultWith(orgAuthMap, orgAuthMap, userAuthBOS);
        }
        Map<Integer, Integer> collect = userAuthBOS.stream().collect(Collectors.toMap(UserAuthBO::getId, UserAuthBO::getDataAuth, (key1, key2) -> key2));
        if (StringUtils.isNotBlank(cacheKey)) {
            cacheService.set(cacheKey, GsonUtils.gsonString(collect));
        }
        return collect;
    }


    /**
     * 获取用户权限
     *
     * @param user        用户
     * @param userAuthBOS 权限载体集合
     * @param dataType    数据类型
     * @return
     */
    private Map<Integer, Integer> getUserAuthMapAllSite(CoreUser user, List<UserAuthBO> userAuthBOS, int dataType) {
        //得到用户的站点权限配置
        List<SysAuthData> userAuth = getList(null, null, user.getId(), null, null, dataType);
        Map<Integer, Integer> userMap = userAuth.stream().collect(Collectors.toMap(SysAuthData::getDataId, SysAuthData::getDataAuth, (key1, key2) -> key2));
        //用户自身有权限则以自身配置的为准，否则如有配置角色则以角色，否则取组织的
        if (!userAuth.isEmpty()) {
            //判断用户是否属于顶级组织,顶层有所有权限无需过滤计算
            if (user.getOrg().isTop()) {
                for (Map.Entry<Integer, Integer> entry : userMap.entrySet()) {
                    userAuthBOS.add(new UserAuthBO(entry.getKey(), entry.getValue()));
                }
            } else {
                //得到组织的站点权限配置,这里得到当前用户的全部组织
                Map<Integer, Integer> orgMap = getOrgAuthMapAllSite(user.getOrg(), false, dataType);
                //权限取与运算
                resultWith(userMap, orgMap, userAuthBOS);
            }
        } else if (!user.getRoles().isEmpty()) {
            //如果用户有角色
            for (CoreRole role : user.getRoles()) {
                getRoleAuthMapAllSite(role, userAuthBOS, dataType);
            }
        } else {
            //否则继承所属组织的权限
            Map<Integer, Integer> orgAuthMap = getOrgAuthMapAllSite(user.getOrg(), true, dataType);
            resultWith(orgAuthMap, orgAuthMap, userAuthBOS);
        }
        //然后再以站点分组，分组里面的数据做或运算
        Map<Integer, List<UserAuthBO>> collect = userAuthBOS.stream().collect(Collectors.groupingBy(UserAuthBO::getId));
        return resultOr(collect);
    }

    /**
     * 栏目类和文档类获取站点内栏目的权限map
     *
     * @return
     */
    private Map<Integer, Integer> getUserAuthMapBySite(Map<Integer, Integer> userAuthMap, int dataType, Integer siteId) {
        Map<Integer, Integer> filterUserAuthMap = new HashMap<>();
        Set<Integer> keySet = userAuthMap.keySet();
        if (SITE_TYPE != dataType) {
            for (Integer key : keySet) {
                Channel channel = channelService.findById(key);
                if (channel != null && channel.getSiteId().equals(siteId)) {
                    filterUserAuthMap.put(key, userAuthMap.get(key));
                }
                //增量配置
                if (-key == siteId) {
                    filterUserAuthMap.put(key, userAuthMap.get(key));
                }
            }
        }
        return filterUserAuthMap;
    }

    /**
     * 获取用户权限
     *
     * @param user        用户
     * @param userAuthBOS 权限载体集合
     * @param dataType    数据类型
     * @param siteId      站点ID 栏目类文档类必有，否则新增栏目类权限无法区分
     * @return
     */
    private Map<Integer, Integer> getUserAuthMap(CoreUser user, List<UserAuthBO> userAuthBOS, int dataType, Integer siteId) {
        //得到用户的站点权限配置
        List<SysAuthData> userAuth = getList(null, null, user.getId(), null, null, dataType);
        Map<Integer, Integer> userMap = userAuth.stream().collect(Collectors.toMap(SysAuthData::getDataId, SysAuthData::getDataAuth, (key1, key2) -> key2));
        //用户自身有权限则以自身配置的为准，否则如有配置角色则以角色，否则取组织的
        if (!userAuth.isEmpty()) {
            //判断用户是否属于顶级组织,顶层有所有权限无需过滤计算
            if (user.getOrg().isTop()) {
                for (Map.Entry<Integer, Integer> entry : userMap.entrySet()) {
                    userAuthBOS.add(new UserAuthBO(entry.getKey(), entry.getValue()));
                }
            } else {
                //得到组织的站点权限配置,这里得到当前用户的全部组织
                Map<Integer, Integer> orgMap = getOrgAuthMap(user.getOrg(), false, dataType, siteId);
                //权限取与运算
                resultWith(userMap, orgMap, userAuthBOS);
            }
        } else if (!user.getRoles().isEmpty()) {
            //如果用户有角色则读取角色权限
            for (CoreRole role : user.getRoles()) {
                getRoleAuthMap(role, userAuthBOS, dataType, siteId);
            }
        } else {
            //否则继承所属组织的权限
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(user.getOrg(), true, dataType, siteId);
            resultWith(orgAuthMap, orgAuthMap, userAuthBOS);
        }
        //然后再以站点分组，分组里面的数据做或运算
        Map<Integer, List<UserAuthBO>> collect = userAuthBOS.stream().collect(Collectors.groupingBy(UserAuthBO::getId));
        return resultOr(collect);
    }


    /**
     * 处理增量站点
     **/
    void dealNewSite(List<SiteDataPermTree> sites, Map<Integer, Integer> selfPermMap, Map<Integer, Integer> parentPermMap,
                     Map<Integer, Integer> userPermMap, boolean isTopOrg) {
        List<CmsDataPermVo.MiniDataUnit> rowDatas = CollUtil.newArrayList();
        SiteDataPermTree vo = new SiteDataPermTree();
        vo.setName("增量站点");
        vo.setParentId(null);
        vo.setId(0);
        vo.setChildren(Collections.emptyList());
        if (isTopOrg) {
            //顶层是固定的有权限不可分配
            String s = AuthUtil.toBinary(SITE_ALL_SELECT, SITE_DIGITS);
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                rowDatas.add(new CmsDataPermVo.MiniDataUnit(i + 1, false, true));
            }
        } else {
            Integer userPermVal = userPermMap.get(0);
            String su = AuthUtil.toBinary(userPermVal, SITE_DIGITS);
            char[] suc = su.toCharArray();
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
            Integer parentPermVal = 0;
            //父层都没有配置
            if (parentPermMap.isEmpty()) {
                parentPermVal = selfPermMap.isEmpty() ? AuthConstant.SITE_ALL_SELECT : 0;
            }
            //父层有配置未包含则是0 没有权限
            if (parentPermMap.get(0) != null) {
                parentPermVal = parentPermMap.get(0);
            }
            //得到之前保存的勾选数据
            Integer permVal = selfPermMap.get(0);
            //勾选的数据与限制数据做与运行，得到可以勾选的数据
            String s = AuthUtil.toBinary(permVal & parentPermVal, SITE_DIGITS);
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                //原来的操作值从1开始 所以+1
                rowDatas.add(new CmsDataPermVo.MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
            }
        }
        vo.setRowDatas(rowDatas);
        sites.add(vo);
    }

    /**
     * 处理增量栏目
     **/
    void dealChannelForContent(List<AdminChildContentDataVO> contents, int select, boolean flag) {
        Map<String, DataBaseVO> map = new HashMap<>(9);
        AdminChildContentDataVO vo = new AdminChildContentDataVO();
        vo.setChannelName("增量栏目");
        vo.setParentId(null);
        vo.setChannelId(0);
        vo.setChildren(Collections.emptyList());
        vo.setFlag(false);
        String s1 = AuthUtil.toBinary(select, CONTENT_DIGITS);
        char[] sc = s1.toCharArray();
        for (int i = 0; i < sc.length; i++) {
            if (flag) {
                map.put("key" + i, new DataBaseVO(i, sc[i], '1'));
            } else {
                map.put("key" + i, new DataBaseVO(i, sc[i], sc[i]));
            }
        }
        vo.setDatas(map);
        contents.add(vo);
    }

    void dealNewSiteForChannel(List<ChannelDataPermTree> channels, Map<Integer, Integer> selfPermMap, Map<Integer, Integer> parentPermMap,
                               Map<Integer, Integer> userPermMap, boolean isTopOrg, Integer siteId, Integer dataType) {
        ChannelDataPermTree vo = new ChannelDataPermTree();
        vo.setName("增量栏目");
        vo.setParentId(null);
        //vo.setSiteId();
        vo.setChildren(Collections.emptyList());
        List<CmsDataPermVo.MiniDataUnit> rowDatas = CollUtil.newArrayList();
        if (isTopOrg) {
            //顶层是固定的有权限不可分配
            String s = AuthUtil.toBinary(CHANNEL_ALL_SELECT, CHANNEL_PERMISSION_DIGITS);
            if (CONTENT_TYPE == dataType) {
                s = AuthUtil.toBinary(CONTENT_ALL_SELECT, CONTENT_DIGITS);
            }
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                rowDatas.add(new CmsDataPermVo.MiniDataUnit(i + 1, false, true));
            }
        } else {
            //栏目类 的增量栏目配置key采用 -siteID
            Integer userPermVal = userPermMap.get(-siteId);
            Integer permLen = CHANNEL_PERMISSION_DIGITS;
            Integer permAllSelect = CHANNEL_ALL_SELECT;
            if (CONTENT_TYPE == dataType) {
                permLen = CONTENT_DIGITS;
                permAllSelect = CONTENT_ALL_SELECT;
            }
            String su = AuthUtil.toBinary(userPermVal, permLen);
            char[] suc = su.toCharArray();
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
            Integer parentPermVal = 0;
            //父层都没有配置
            if (parentPermMap.isEmpty()) {
                parentPermVal = selfPermMap.isEmpty() ? permAllSelect : 0;
            }
            //父层有配置未包含则是0 没有权限
            if (parentPermMap.get(-siteId) != null) {
                parentPermVal = parentPermMap.get(-siteId);
            }
            //得到之前保存的勾选数据,如果取到了则是配置了，否则如果为空则一定是继承的给1，selfPermMap不为空又没有取到则没有权限
            Integer permVal = (selfPermMap.get(-siteId) != null) ? selfPermMap.get(-siteId) : (selfPermMap.isEmpty() ? permAllSelect : 0);
            //勾选的数据与限制数据做与运行，得到可以勾选的数据
            String s = AuthUtil.toBinary(permVal & parentPermVal, permLen);
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                //原来的操作值从1开始 所以+1
                rowDatas.add(new CmsDataPermVo.MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
            }
        }
        vo.setRowDatas(rowDatas);
        channels.add(vo);
    }

    @Override
    public void roleSite(DataRoleDTO dto) throws GlobalException {
        CoreRole role = roleService.findById(dto.getRoleId());
        if (role != null) {
            List<SysAuthData> dataList = new ArrayList<>();
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, dto.getRoleId(), null, null, null, SITE_TYPE);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            Map<Integer, Integer> roleAuthMap = getDtoAuthMap(dto.getSites());
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(role.getOrg(), false, SITE_TYPE, null);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(roleAuthMap, orgAuthMap);
            if (authEqOwnerOrg && !list.isEmpty()) {
                super.physicalDeleteInBatch(list);
            } else {
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增
                    save(dto.getSites(), dataList, dto.getRoleId(), AuthEnum.ROLE_TYPE);
                } else {
                    dealWithData(covertBo(dto.getSites()), list, dataList, role.getOrgid(), dto.getRoleId(), AuthEnum.ROLE_TYPE);
                }
                super.saveAll(dataList);
            }
            //处理角色站点类缓存
            for (DataSiteBaseDTO site : dto.getSites()) {
                dealWithCacheForRolesSite(Arrays.asList(role.getId()), site.getSiteId());
            }
            clearCacheForRoleSiteAuth(role.getId());
            //用户的权限可能来自角色
            ThreadUtil.execute(() -> {
                for (CoreUser user : role.getUsers()) {
                    clearCacheForUserSiteAuth(user.getOrgId(), user.getId());
                }
            });
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public CmsDataPermVo roleSiteDetails(Integer roleId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<SiteDataPermTree> siteDataPermTrees = CollUtil.newArrayList();
        vo.setDataIds(siteDataPermTrees);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        //得到站点树
        String data = getSiteData();
        if (StringUtils.isBlank(data)) {
            return vo;
        }
        CoreRole role = roleService.findById(roleId);
        if (role != null) {
            vo.setManagerAble(role.getDeleteAble());
            //得到勾选权限数据
            List<SysAuthData> roleAuthDataList = dao.getList(null, roleId, null, null, null, SITE_TYPE);
            //有独立的权限数据则是单独分配了
            if (!roleAuthDataList.isEmpty()) {
                vo.setAssigned(true);
            }
            Map<Integer, Integer> rolePermMap = getRoleAuthMap(role, CollUtil.newArrayList(), SITE_TYPE, null);
            //得到限制权限数据
            CmsOrg org = role.getOrg();
            List<SiteDataPermTree> dataPermTrees = CollUtil.newArrayList();
            List<AdminSiteTreeVO> siteTreeVos = GsonUtils.gsonToList(data, AdminSiteTreeVO.class);
            //得到组织权限数据
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(role.getOrg(), false, SITE_TYPE, null);
            Map<Integer, Integer> userAuthMap = getSiteAuth(SystemContextUtils.getCurrentUsername());
            //用户有权限还得父层有权限才可分配，此处与运算，userAuthMap 主要用于 convert方法判断是否可分配
            resultWith(userAuthMap, orgAuthMap);
            convert(siteTreeVos, rolePermMap, orgAuthMap, userAuthMap, dataPermTrees, org.isTop());
            //树排序
            dataPermTrees = SiteDataPermTree.sort(dataPermTrees);
            dealNewSite(dataPermTrees, rolePermMap, orgAuthMap, userAuthMap, false);
            vo.setDataIds(dataPermTrees);
        }
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void userSite(DataUserDTO dto) throws GlobalException {
        Map<String, Integer> map = new HashMap<>(10);
        List<SysAuthData> dataList = new ArrayList<>();
        CoreUser user = userService.findById(dto.getUserId());
        if (user != null) {
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            Map<Integer, Integer> userAuthMap = getDtoAuthMap(dto.getSites());
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(user.getOrg(), true, SITE_TYPE, null);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(userAuthMap, orgAuthMap);
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, null, dto.getUserId(), null, null, SITE_TYPE);
            if (authEqOwnerOrg && !list.isEmpty()) {
                super.physicalDeleteInBatch(list);
            } else {
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增
                    save(dto.getSites(), dataList, dto.getUserId(), AuthEnum.USER_TYPE);
                } else {
                    dealWithData(covertBo(dto.getSites()), list, dataList, user.getOrgId(), dto.getUserId(), AuthEnum.USER_TYPE);
                }
                super.saveAll(dataList);
                //权限值变更的 处理修改缓存
                for (SysAuthData authData : dataList) {
                    map.put(CacheConstant.getSiteAuthUserKey(user.getOrgId(), dto.getUserId(), authData.getDataId()),
                            authData.getDataAuth());
                }
            }
            clearCacheForUserSiteAuth(user.getOrgId(), user.getId());
            cacheService.multiSet(map);
        }
    }

    @Override
    public CmsDataPermVo userSiteDetails(Integer userId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<SiteDataPermTree> siteDataPermTrees = CollUtil.newArrayList();
        vo.setDataIds(siteDataPermTrees);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        //得到站点树
        String data = getSiteData();
        if (StringUtils.isBlank(data)) {
            return vo;
        }
        CoreUser user = userService.findById(userId);
        if (user != null) {
            vo.setManagerAble(user.getDeleteAble());
            //得到勾选权限数据
            List<SysAuthData> roleAuthDataList = dao.getList(null, null, userId, null, null, SITE_TYPE);
            //有独立的权限数据则是单独分配了
            if (!roleAuthDataList.isEmpty()) {
                vo.setAssigned(true);
            }
            Map<Integer, Integer> userAuthMap = getUserAuthMap(user, CollUtil.newArrayList(), SITE_TYPE, null);
            //得到限制权限数据
            CmsOrg org = user.getOrg();
            List<SiteDataPermTree> dataPermTrees = CollUtil.newArrayList();
            List<AdminSiteTreeVO> siteTreeVos = GsonUtils.gsonToList(data, AdminSiteTreeVO.class);
            //得到组织权限数据
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(user.getOrg(), false, SITE_TYPE, null);
            Map<Integer, Integer> currUserAuthMap = getSiteAuth(SystemContextUtils.getCurrentUsername());
            //用户有权限还得父层有权限才可分配，此处与运算，currUserAuthMap 主要用于 convert方法判断是否可分配
            resultWith(currUserAuthMap, orgAuthMap);
            convert(siteTreeVos, userAuthMap, orgAuthMap, currUserAuthMap, dataPermTrees, org.isTop());
            //树排序
            dataPermTrees = SiteDataPermTree.sort(dataPermTrees);
            dealNewSite(dataPermTrees, userAuthMap, orgAuthMap, currUserAuthMap, false);
            vo.setDataIds(dataPermTrees);
        }
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orgContent(DataOrgContentDTO dto) throws GlobalException {
        CmsOrg org = orgService.findById(dto.getOrgId());
        if (org.getParent() != null) {
            //原有权限 后面比较
            Map<Integer, Integer> orgAuthMap = getOrgAuthMapAllSite(org, true, CONTENT_TYPE);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            orgAuthMap.putAll(getDtoContentAuthMap(dto.getContents(), dto.getSiteId()));
            Map<Integer, Integer> parentOrgAuthMap = getOrgAuthMapAllSite(org.getParent(), true, CONTENT_TYPE);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(orgAuthMap, parentOrgAuthMap);
            //得到权限数据
            List<SysAuthData> list = dao.getList(Collections.singletonList(dto.getOrgId()), null, null, null, null, CONTENT_TYPE);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                Map<String, Integer> map = new HashMap<>(10);
                List<SysAuthData> dataList = new ArrayList<>();
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveContent(dto.getContents(), dataList, dto.getOrgId(),
                            dto.getSiteId(), AuthEnum.ORG_TYPE);
                } else {
                    list = dao.getList(Collections.singletonList(dto.getOrgId()), null, null, dto.getSiteId(), null, CONTENT_TYPE);
                    dealWithDataForContent(covertBoForContent(dto.getContents()), list, dataList, null,
                            dto.getOrgId(), AuthEnum.ORG_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
                //处理修改缓存
                for (SysAuthData authData : dataList) {
                    map.put(CacheConstant.getContentAuthOrgKey(authData.getOrgId(), authData.getSiteId(), authData.getDataId()),
                            authData.getDataAuth());
                }
                cacheService.multiSet(map);
            }
            clearCacheForOrgContentAuth(org.getId());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void roleContent(DataRoleContentDTO dto) throws GlobalException {
        CoreRole role = roleService.findById(dto.getRoleId());
        if (role != null) {
            //原有权限 后面比较
            Map<Integer, Integer> selfAuthMap = getRoleAuthMapAllSite(role, CollUtil.newArrayList(), CONTENT_TYPE);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            selfAuthMap.putAll(getDtoContentAuthMap(dto.getContents(), dto.getSiteId()));
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, dto.getRoleId(), null, null, null, CONTENT_TYPE);
            Map<Integer, Integer> parentOrgAuthMap = getOrgAuthMapAllSite(role.getOrg(), false, CONTENT_TYPE);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(selfAuthMap, parentOrgAuthMap);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                list = dao.getList(null, dto.getRoleId(), null, dto.getSiteId(), null, CONTENT_TYPE);
                Map<String, Integer> map = new HashMap<>(10);
                List<SysAuthData> dataList = new ArrayList<>();
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveContent(dto.getContents(), dataList, dto.getRoleId(),
                            dto.getSiteId(), AuthEnum.ROLE_TYPE);
                } else {
                    dealWithDataForContent(covertBoForContent(dto.getContents()), list, dataList, role.getOrgid(),
                            dto.getRoleId(), AuthEnum.ROLE_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
                //处理修改缓存
                for (SysAuthData authData : dataList) {
                    map.put(CacheConstant.getContentAuthRoleKey(role.getOrgid(), authData.getRoleId(),
                                    authData.getSiteId(), authData.getDataId()),
                            authData.getDataAuth());
                }
                cacheService.multiSet(map);
            }
            clearCacheForRoleContentAuth(role.getId());
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void userContent(DataUserContentDTO dto) throws GlobalException {
        CoreUser user = userService.findById(dto.getUserId());
        if (user != null) {
            //原有权限 后面比
            Map<Integer, Integer> authMap = getUserAuthMapAllSite(user, CollUtil.newArrayList(), CONTENT_TYPE);
            //较预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            authMap.putAll(getDtoContentAuthMap(dto.getContents(), dto.getSiteId()));
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, null, dto.getUserId(), null, null, CONTENT_TYPE);
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(user.getOrg(), false, CONTENT_TYPE, dto.getSiteId());
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(authMap, orgAuthMap);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                Map<String, Integer> map = new HashMap<>(10);
                List<SysAuthData> dataList = new ArrayList<>();
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveContent(dto.getContents(), dataList, dto.getUserId(), dto.getSiteId(), AuthEnum.USER_TYPE);
                } else {
                    list = dao.getList(null, null, dto.getUserId(), dto.getSiteId(), null, CONTENT_TYPE);
                    dealWithDataForContent(covertBoForContent(dto.getContents()), list, dataList, user.getOrgId(),
                            dto.getUserId(), AuthEnum.USER_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
                //用户权限变动不会影响有影响，这里只需要处理修改缓存
                for (SysAuthData authData : dataList) {
                    map.put(CacheConstant.getContentAuthUserKey(user.getOrgId(), authData.getUserId(),
                                    authData.getSiteId(), authData.getDataId()),
                            authData.getDataAuth());
                }
                cacheService.multiSet(map);
            }
            clearCacheForUserContentAuth(user.getOrgId(), user.getId());
        }
    }

    /**
     * 统一处理新增修改数据---内容权限
     **/
    private void dealWithDataForContent(List<DataContentBaseBO> contents, List<SysAuthData> list, List<SysAuthData> dataList,
                                        Integer orgId, Integer id, AuthEnum authEnum, Integer siteId) {
        if (!contents.isEmpty()) {
            //前端之前数据这里是null，这里null统一设置-siteId，为了后续统一处理
            List<DataContentBaseBO> newDataAuthKey = contents.stream().filter(x -> x.getChannelId() == null).collect(Collectors.toList());
            if (newDataAuthKey.size() > 1) {
                newDataAuthKey = newDataAuthKey.subList(0, 1);
            }
            for (DataContentBaseBO c : newDataAuthKey) {
                c.setChannelId(-siteId);
            }
            contents = contents.stream().filter(x -> x.getChannelId() != null).collect(Collectors.toList());
        }
        //如果存在，做权限对比，只修改有改动的权限
        //1.先得到传来的数据
        List<String> list1 = contents.stream().map(x -> x.getChannelId()
                        + StrUtil.UNDERLINE
                        + AuthConstant.getAuthInt(x.getDatas().values()))
                .collect(Collectors.toList());
        //2.得到数据库的数据
        List<String> list2 = list.stream().map(x -> x.getDataId()
                        + StrUtil.UNDERLINE
                        + x.getDataAuth())
                .collect(Collectors.toList());
        //3.对比，得到修改了的数据
        list1.removeAll(list2);
        for (String x : list1) {
            String[] split = x.split(StrUtil.UNDERLINE);
            if (AuthEnum.ORG_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndOrgIdAndDataType(Integer.valueOf(split[0]), id, CONTENT_TYPE);
                if (data != null) {
                    Integer oldAuth = data.getDataAuth();
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                    //处理缓存，如果传值小于缓存值，则需要处理
                    if (Integer.parseInt(split[1]) < oldAuth) {
                        dealWithCacheForOrgContent(orgId, siteId, Integer.valueOf(split[0]));
                    }
                } else {
                    SysAuthData sysAuthData = new SysAuthData();
                    sysAuthData.setOrgId(id);
                    sysAuthData.setSiteId(siteId);
                    sysAuthData.setDataId(Integer.valueOf(split[0]));
                    sysAuthData.setDataType(CONTENT_TYPE);
                    sysAuthData.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(sysAuthData);
                }
            } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndRoleIdAndDataType(Integer.valueOf(split[0]), id, CONTENT_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                } else {
                    SysAuthData sysAuthData = new SysAuthData();
                    sysAuthData.setRoleId(id);
                    sysAuthData.setSiteId(siteId);
                    sysAuthData.setDataId(Integer.valueOf(split[0]));
                    sysAuthData.setDataType(CONTENT_TYPE);
                    sysAuthData.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(sysAuthData);
                }
            } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndUserIdAndDataType(Integer.valueOf(split[0]), id, CONTENT_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(data);
                } else {
                    SysAuthData sysAuthData = new SysAuthData();
                    sysAuthData.setUserId(id);
                    sysAuthData.setSiteId(siteId);
                    sysAuthData.setDataId(Integer.valueOf(split[0]));
                    sysAuthData.setDataType(CONTENT_TYPE);
                    sysAuthData.setDataAuth(Integer.valueOf(split[1]));
                    dataList.add(sysAuthData);
                }
            }
        }
        //4.删除多余的数据
        deleteCacheAndEntityForContent(contents, list, orgId, id, siteId, authEnum);
    }

    /**
     * 统一处理新增修改数据---栏目权限
     **/
    private void dealWithDataForChannel(List<DataChannelBaseBO> channels, List<SysAuthData> list, List<SysAuthData> dataList,
                                        Integer orgId, Integer id, AuthEnum authEnum, Integer siteId) {
        if (!channels.isEmpty()) {
            //前端之前数据这里是null，这里null统一设置-siteId，为了后续统一处理
            List<DataChannelBaseBO> newDataAuthKey = channels.stream().filter(x -> x.getChannelId() == null).collect(Collectors.toList());
            if (newDataAuthKey.size() > 1) {
                newDataAuthKey = newDataAuthKey.subList(0, 1);
            }
            for (DataChannelBaseBO c : newDataAuthKey) {
                c.setChannelId(-siteId);
            }
            channels = channels.stream().filter(x -> x.getChannelId() != null).collect(Collectors.toList());
        }
        //如果存在，做权限对比，只修改有改动的权限
        //1.先得到传来的数据
        List<String> list1 = channels.stream().map(x -> x.getChannelId()
                        + StrUtil.UNDERLINE
                        + AuthConstant.getAuthInt(x.getDatas().values()))
                .collect(Collectors.toList());
        //2.得到数据库的数据
        List<String> list2 = list.stream().map(x -> x.getDataId()
                        + StrUtil.UNDERLINE
                        + x.getDataAuth())
                .collect(Collectors.toList());
        //3.对比，得到修改了的数据
        list1.removeAll(list2);
        //todo xiaohui 循环里面套了查询
        for (String x : list1) {
            String[] split = x.split(StrUtil.UNDERLINE);
            String channelId = split[0];
            String authNum = split[1];
            if (AuthEnum.ORG_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndOrgIdAndDataType(Integer.valueOf(channelId), id, CHANNEL_TYPE);
                if (data != null) {
                    Integer oldAuth = data.getDataAuth();
                    data.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(data);
                    //处理缓存，如果传值小于缓存值，则需要处理
                    if (Integer.parseInt(authNum) < oldAuth) {
                        dealWithCacheForOrgChannel(orgId, siteId, Integer.valueOf(channelId));
                    }
                } else {
                    SysAuthData authData = new SysAuthData();
                    authData.setOrgId(id);
                    authData.setSiteId(siteId);
                    authData.setDataId(Integer.valueOf(channelId));
                    authData.setDataType(CHANNEL_TYPE);
                    authData.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(authData);
                }
            } else if (AuthEnum.ROLE_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndRoleIdAndDataType(Integer.valueOf(channelId), id, CHANNEL_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(data);
                } else {
                    SysAuthData authData = new SysAuthData();
                    authData.setRoleId(id);
                    authData.setSiteId(siteId);
                    authData.setDataId(Integer.valueOf(channelId));
                    authData.setDataType(CHANNEL_TYPE);
                    authData.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(authData);
                }
            } else if (AuthEnum.USER_TYPE.equals(authEnum)) {
                SysAuthData data = dao.findByDataIdAndUserIdAndDataType(Integer.valueOf(channelId), id, CHANNEL_TYPE);
                if (data != null) {
                    data.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(data);
                } else {
                    SysAuthData authData = new SysAuthData();
                    authData.setUserId(id);
                    authData.setSiteId(siteId);
                    authData.setDataId(Integer.valueOf(channelId));
                    authData.setDataType(CHANNEL_TYPE);
                    authData.setDataAuth(Integer.valueOf(authNum));
                    dataList.add(authData);
                }
            }
        }
        //4.删除多余的数据
        deleteCacheAndEntityForChannel(channels, list, orgId, id, siteId, authEnum);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orgChannel(DataChannelDTO dto) throws GlobalException {
        Map<String, Integer> map = new HashMap<>(10);
        List<SysAuthData> dataList = new ArrayList<>();
        CmsOrg org = orgService.findById(dto.getId());
        if (org.getParent() != null) {
            //原有权限 后面比较
            Map<Integer, Integer> orgAuthMap = getOrgAuthMapAllSite(org, true, CHANNEL_TYPE);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            orgAuthMap.putAll(getDtoChannelAuthMap(dto.getChannels(), dto.getSiteId()));
            Map<Integer, Integer> parentOrgAuthMap = getOrgAuthMapAllSite(org.getParent(), true, CHANNEL_TYPE);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(orgAuthMap, parentOrgAuthMap);
            //得到权限数据，这里应该全站的数据
            List<SysAuthData> list = dao.getList(Collections.singletonList(dto.getId()), null, null, null, null, CHANNEL_TYPE);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveChannel(dto.getChannels(), dataList, dto.getId(),
                            dto.getSiteId(), AuthEnum.ORG_TYPE);
                } else {
                    //修改的情况是针对的本站点的
                    list = dao.getList(Collections.singletonList(dto.getId()), null, null, dto.getSiteId(), null, CHANNEL_TYPE);
                    dealWithDataForChannel(covertBoForChannel(dto.getChannels()), list, dataList, dto.getId(),
                            dto.getId(), AuthEnum.ORG_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
            }
            for (DataChannelBaseDTO channelBaseDTO : dto.getChannels()) {
                dealWithCacheForChannelOrg(Arrays.asList(org.getId()), dto.getSiteId(), channelBaseDTO.getChannelId());
            }
            clearCacheForOrgChannelAuth(org.getId());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public CmsDataPermVo orgChannelDetails(Integer orgId, Integer siteId, Integer dataType, boolean lazy, Integer parentId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<ChannelDataPermTree> channelDataPermTrees = CollUtil.newArrayList();
        CmsOrg org = orgService.findById(orgId);
        CmsSite site = siteService.findById(siteId);
        CmsDataPermVo.ChannelRow channelRow = new CmsDataPermVo.ChannelRow();
        channelRow.setSite(site);
        channelRow.setUnits(channelDataPermTrees);
        vo.setMoreDataIds(channelRow);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        //特殊处理顶级组织
        if (org.isTop()) {
            vo.setManagerAble(false);
            channelRow.setUnits(getOrgChannelPermTree(org, siteId, dataType, lazy, parentId));
            return vo;
        }
        vo.setManagerAble(org.getDeleteAble());
        List<SysAuthData> orgAuthDataList = dao.getList(Collections.singletonList(orgId), null, null, null, null, dataType);
        //有独立的权限数据则是单独分配了
        if (!orgAuthDataList.isEmpty()) {
            vo.setAssigned(true);
        }
        //得到栏目树
        channelRow.setUnits(getOrgChannelPermTree(org, siteId, dataType, lazy, parentId));
        return vo;
    }

    @Override
    public List<Integer> getOrgChannelSite(Integer id, Integer dataType) {
        CmsOrg org = orgService.findById(id);
        List<CmsSite> cmsSites = siteService.findAll(false, true);
        List<Integer> hasAssignPermSiteIds = CollUtil.newArrayList();
        for (CmsSite s : cmsSites) {
            //取得组织权限，如果权限值不是都是0，则说明有该站点的部分栏目权限
            List<ChannelDataPermTree> channelDataPermTrees = getOrgChannelPermTree(org, s.getId(), dataType, false, null);
            boolean hasSiteChannelPerm = false;
            for (ChannelDataPermTree d : channelDataPermTrees) {
                for (MiniDataUnit u : d.getRowDatas()) {
                    if (u.getHasPerm()) {
                        hasSiteChannelPerm = true;
                        break;
                    }
                }
                if (hasSiteChannelPerm) {
                    break;
                }
            }
            if (hasSiteChannelPerm) {
                hasAssignPermSiteIds.add(s.getId());
            }
        }
        return hasAssignPermSiteIds;
    }

    private List<ChannelDataPermTree> getOrgChannelPermTree(CmsOrg org, Integer siteId, Integer dataType, boolean lazy, Integer parentId) {
        List<ChannelDataPermTree> dataPermTrees = CollUtil.newArrayList();
        String data = getChannelData(siteId);
        if (StringUtils.isBlank(data)) {
            return dataPermTrees;
        }
        List<AdminChannelTreeVO> voList = GsonUtils.gsonToList(data, AdminChannelTreeVO.class);
        //特殊处理顶级组织
        if (org.isTop()) {
            topOrgAndAdminForChannel(voList, dataPermTrees, dataType);
            //转换树
            dataPermTrees = ChannelDataPermTree.converToTree(dataPermTrees);
            //处理增量栏目
            dealNewSiteForChannel(dataPermTrees, null, null, null, true, siteId, dataType);
            return dataPermTrees;
        }
        CmsOrg parentOrg = org.getParent();
        //取所有上层所有父节点权限
        Map<Integer, Integer> parentAuthMap = getOrgAuthMap(parentOrg, true, dataType, siteId);
        //得到权限数据
        Map<Integer, Integer> selfAuthMap = getOrgAuthMap(org, true, dataType, siteId);
        return getChannelPermTree(selfAuthMap, parentAuthMap, siteId, org.getParent().isTop(), dataType, lazy, parentId, true);
    }

    private List<ChannelDataPermTree> getRoleChannelPermTree(CoreRole role, Integer siteId, Integer dataType, boolean lazy, Integer parentId) {
        CmsOrg parentOrg = role.getOrg();
        //取所有上层所有父节点权限
        Map<Integer, Integer> parentAuthMap = getOrgAuthMap(parentOrg, false, dataType, siteId);
        //得到权限数据
        Map<Integer, Integer> selfAuthMap = getRoleAuthMap(role, CollUtil.newArrayList(), dataType, siteId);
        return getChannelPermTree(selfAuthMap, parentAuthMap, siteId, false, dataType, lazy, parentId, true);
    }

    private List<ChannelDataPermTree> getUserChannelPermTree(CoreUser user, Integer siteId,
                                                             Integer dataType, boolean lazy, Integer parentId) {
        CmsOrg parentOrg = user.getOrg();
        //取所有上层所有父节点权限
        Map<Integer, Integer> parentAuthMap = getOrgAuthMap(parentOrg, false, dataType, siteId);
        //得到权限数据
        Map<Integer, Integer> selfAuthMap = getUserAuthMap(user, CollUtil.newArrayList(), dataType, siteId);
        return getChannelPermTree(selfAuthMap, parentAuthMap, siteId, false, dataType, lazy, parentId, true);
    }

    public List<ChannelDataPermTree> getUserChannelPermTreeNoNewSitePerm(CoreUser user, Integer siteId,
                                                                         Integer dataType, boolean lazy, Integer parentId) {
        CmsOrg parentOrg = user.getOrg();
        //取所有上层所有父节点权限
        Map<Integer, Integer> parentAuthMap = getOrgAuthMap(parentOrg, false, dataType, siteId);
        //得到权限数据
        Map<Integer, Integer> selfAuthMap = getUserAuthMap(user, CollUtil.newArrayList(), dataType, siteId);
        return getChannelPermTree(selfAuthMap, parentAuthMap, siteId, false, dataType, lazy, parentId, false);
    }

    private List<ChannelDataPermTree> getChannelPermTree(Map<Integer, Integer> selfAuthMap, Map<Integer, Integer> parentAuthMap,
                                                         Integer siteId, Boolean parentOrgIsTop, Integer dataType, boolean lazy, Integer parentId, boolean withNewSitePerm) {
        List<ChannelDataPermTree> dataPermTrees = CollUtil.newArrayList();
        String data = getChannelData(siteId);
        if (StringUtils.isBlank(data)) {
            return dataPermTrees;
        }
        List<AdminChannelTreeVO> voList = GsonUtils.gsonToList(data, AdminChannelTreeVO.class);
        if (lazy) {
            voList = filterChannelTreeByParentId(voList, parentId, CollUtil.newArrayList());
        }
        //这里是取的用户所有站群的栏目权限
        Map<Integer, Integer> userAuthMap = getChannelAuth(SystemContextUtils.getCurrentUsername(), dataType);
        //用户有权限还得父层有权限才可分配，此处与运算，userAuthMap 主要用于 convert方法判断是否可分配
        resultWith(userAuthMap, parentAuthMap);
        convertForChannel(voList, selfAuthMap, parentAuthMap, userAuthMap, dataPermTrees, parentOrgIsTop, dataType);
        if (siteId != null) {
            dataPermTrees = dataPermTrees.stream().filter(p -> p.getSiteId().equals(siteId)).collect(Collectors.toList());
        }
        //处理新站点栏目权限
        if (withNewSitePerm) {
            //树排序
            dataPermTrees = ChannelDataPermTree.converToTree(dataPermTrees);
            dealNewSiteForChannel(dataPermTrees, selfAuthMap, parentAuthMap, userAuthMap, false, siteId, dataType);
        }
        return dataPermTrees;
    }

    private List<AdminChannelTreeVO> filterChannelTreeByParentId(List<AdminChannelTreeVO> voList, Integer parentId, List<AdminChannelTreeVO> result) {
        if (!voList.isEmpty()) {
            //获取顶层
            if (ObjectUtil.isNull(parentId)) {
                for (AdminChannelTreeVO vo : voList) {
                    vo.setChildren(CollUtil.newArrayList());
                }
                return voList;
            }
            //有多层结构，需要每层都去遍历添加到结果中
            for (AdminChannelTreeVO vo : voList) {
                if (parentId.equals(vo.getParentId())) {
                    for (AdminChannelTreeVO v : voList) {
                        v.setChildren(CollUtil.newArrayList());
                    }
                    result.addAll(voList);
                    return result;
                } else {
                    filterChannelTreeByParentId(vo.getChildren(), parentId, result);
                }
            }
        }
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void roleChannel(DataChannelDTO dto) throws GlobalException {
        CoreRole role = roleService.findById(dto.getId());
        if (role != null) {
            Map<String, Integer> map = new HashMap<>(10);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            //原有权限 后面比较
            Map<Integer, Integer> authMap = getRoleAuthMapAllSite(role, CollUtil.newArrayList(), CHANNEL_TYPE);
            authMap.putAll(getDtoChannelAuthMap(dto.getChannels(), dto.getSiteId()));
            List<SysAuthData> dataList = new ArrayList<>();
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, dto.getId(), null, null, null, CHANNEL_TYPE);
            Map<Integer, Integer> orgAuthMap = getOrgAuthMapAllSite(role.getOrg(), false, CHANNEL_TYPE);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(authMap, orgAuthMap);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveChannel(dto.getChannels(), dataList, dto.getId(),
                            dto.getSiteId(), AuthEnum.ROLE_TYPE);
                } else {
                    list = dao.getList(null, dto.getId(), null, dto.getSiteId(), null, CHANNEL_TYPE);
                    dealWithDataForChannel(covertBoForChannel(dto.getChannels()), list, dataList, role.getOrgid(),
                            dto.getId(), AuthEnum.ROLE_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
            }
            for (DataChannelBaseDTO channelBaseDTO : dto.getChannels()) {
                dealWithCacheForChannelRole(Arrays.asList(role.getId()), dto.getSiteId(), channelBaseDTO.getChannelId());
            }
            clearCacheForRoleChannelAuth(role.getId());
            //用户的权限可能来自角色
            ThreadUtil.execute(() -> {
                for (CoreUser user : role.getUsers()) {
                    clearCacheForUserChannelAuth(user.getOrgId(), user.getId());
                    clearCacheForUserContentAuth(user.getOrgId(), user.getId());
                }
            });
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public CmsDataPermVo roleChannelDetails(Integer roleId, Integer siteId, Integer dataType, boolean lazy, Integer parentId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<ChannelDataPermTree> channelDataPermTrees = CollUtil.newArrayList();
        CoreRole role = roleService.findById(roleId);
        CmsSite site = siteService.findById(siteId);
        CmsDataPermVo.ChannelRow channelRow = new CmsDataPermVo.ChannelRow();
        channelRow.setSite(site);
        channelRow.setUnits(channelDataPermTrees);
        vo.setMoreDataIds(channelRow);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        vo.setManagerAble(role.getDeleteAble());
        List<SysAuthData> orgAuthDataList = dao.getList(null, roleId, null, null, null, dataType);
        //有独立的权限数据则是单独分配了
        if (!orgAuthDataList.isEmpty()) {
            vo.setAssigned(true);
        }
        //得到栏目树
        channelRow.setUnits(getRoleChannelPermTree(role, siteId, dataType, lazy, parentId));
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void userChannel(DataChannelDTO dto) throws GlobalException {
        CoreUser user = userService.findById(dto.getId());
        if (user != null) {
            //原有权限 后面比较
            Map<Integer, Integer> authMap = getUserAuthMapAllSite(user, CollUtil.newArrayList(), CHANNEL_TYPE);
            //预处理后的权限数据如果等于所属组织，则清空 认定是想要继承组织
            authMap.putAll(getDtoChannelAuthMap(dto.getChannels(), dto.getSiteId()));
            //得到权限数据
            List<SysAuthData> list = dao.getList(null, null, dto.getId(), null, null, CHANNEL_TYPE);
            Map<Integer, Integer> orgAuthMap = getOrgAuthMapAllSite(user.getOrg(), false, CHANNEL_TYPE);
            Boolean authEqOwnerOrg = AuthUtil.mapCompareIn(authMap, orgAuthMap);
            if (authEqOwnerOrg) {
                super.physicalDeleteInBatch(list);
            } else {
                List<SysAuthData> dataList = new ArrayList<>();
                if (list.isEmpty()) {
                    //如果之前没配置权限，直接新增,递归新增
                    saveChannel(dto.getChannels(), dataList, dto.getId(),
                            dto.getSiteId(), AuthEnum.USER_TYPE);
                } else {
                    list = dao.getList(null, null, dto.getId(), dto.getSiteId(), null, CHANNEL_TYPE);
                    dealWithDataForChannel(covertBoForChannel(dto.getChannels()), list, dataList, user.getOrgId(),
                            dto.getId(), AuthEnum.USER_TYPE, dto.getSiteId());
                }
                super.saveAll(dataList);
            }
            for (DataChannelBaseDTO channelBaseDTO : dto.getChannels()) {
                dealWithCacheForChannelUser(Arrays.asList(user.getId()), dto.getSiteId(), channelBaseDTO.getChannelId());
            }
            clearCacheForUserChannelAuth(user.getOrgId(), user.getId());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public CmsDataPermVo userChannelDetails(Integer userId, Integer siteId, Integer dataType, boolean lazy, Integer parentId) {
        CmsDataPermVo vo = new CmsDataPermVo();
        List<ChannelDataPermTree> channelDataPermTrees = CollUtil.newArrayList();
        CoreUser user = userService.findById(userId);
        CmsSite site = siteService.findById(siteId);
        CmsDataPermVo.ChannelRow channelRow = new CmsDataPermVo.ChannelRow();
        channelRow.setSite(site);
        channelRow.setUnits(channelDataPermTrees);
        vo.setMoreDataIds(channelRow);
        vo.setAssigned(false);
        vo.setManagerAble(true);
        vo.setManagerAble(user.getDeleteAble());
        List<SysAuthData> orgAuthDataList = dao.getList(null, null, userId, null, null, dataType);
        //有独立的权限数据则是单独分配了
        if (!orgAuthDataList.isEmpty()) {
            vo.setAssigned(true);
        }
        //得到栏目树
        channelRow.setUnits(getUserChannelPermTree(user, siteId, dataType, lazy, parentId));
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public SysAuthData findAuth(Integer orgId, Integer roleId, Integer userId, Integer dataId, Integer dataType) {
        List<Integer> ids = null;
        if (orgId != null) {
            ids = Collections.singletonList(orgId);
        }
        List<SysAuthData> list = dao.getList(ids, roleId, userId, null, dataId, dataType);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public List<SysAuthData> findAuths(Integer orgId, Integer roleId, Integer userId, Integer dataType) {
        List<Integer> ids = null;
        if (orgId != null) {
            ids = Collections.singletonList(orgId);
        }
        return dao.getList(ids, roleId, userId, null, null, dataType);
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public int countByUserIdAndDataType(Integer userId, Integer dataType) {
        return dao.countByUserIdAndDataType(userId, dataType);
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public int countByRoleIdAndDataType(Integer roleId, Integer dataType) {
        return dao.countByRoleIdAndDataType(roleId, dataType);
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public int countByOrgIdAndDataType(Integer orgId, Integer dataType) {
        return dao.countByOrgIdAndDataType(orgId, dataType);
    }

    /**
     * 通用与运算
     **/
    private void resultWith(Map<Integer, Integer> self, Map<Integer, Integer> parent, List<UserAuthBO> bo) {
        //权限取与运算
        for (Map.Entry<Integer, Integer> entry : self.entrySet()) {
            //如果限制不存在用户勾选的站点，则忽略
            if (parent.get(entry.getKey()) != null) {
                Integer r = parent.get(entry.getKey());
                UserAuthBO bo1 = new UserAuthBO(entry.getKey(),
                        r & entry.getValue());
                bo.add(bo1);
            }
        }
    }

    /**
     * 通用或运算
     **/
    private Map<Integer, Integer> resultOr(Map<Integer, List<UserAuthBO>> collect) {
        Map<Integer, Integer> boResult = new HashMap<>(16);
        for (Map.Entry<Integer, List<UserAuthBO>> entry : collect.entrySet()) {
            //得到全部的权限，取或运算
            List<Integer> list = entry.getValue().stream().map(UserAuthBO::getDataAuth)
                    .collect(Collectors.toList());
            int sum = 0;
            for (Integer integer : list) {
                sum |= integer;
            }
            boResult.put(entry.getKey(), sum);
        }
        return boResult;
    }

    /**
     * 通用与运算 返回parent与userMap
     **/
    private void resultWith(Map<Integer, Integer> self, Map<Integer, Integer> parent) {
        //权限取与运算
        for (Map.Entry<Integer, Integer> entry : self.entrySet()) {
            //如果限制不存在用户勾选的站点，则忽略
            if (parent.get(entry.getKey()) != null) {
                Integer r = parent.get(entry.getKey());
                self.put(entry.getKey(), r & entry.getValue());
            }
        }
    }

    /**
     * 转换数据，整体逻辑如下
     * 1.得到全部的站点数据
     * 2.如果之前勾了就得到勾了的权限，然后解析得到相应的数据
     * 3.判断是否可选，组织则判断上级组织的勾选范围，角色判断所属组织，用户则判断所属组织
     *
     * @param siteTree      站点数据
     * @param selfPermMap   本身勾选的数据
     * @param parentPermMap 限制数据
     * @param userPermMap   用户的权限数据
     * @param sites         装载结果集合
     * @param parentIsTop   权限继承来源是否顶层组织，是否特殊处理父级是顶层站点的情况
     */
    private void convert(List<AdminSiteTreeVO> siteTree, Map<Integer, Integer> selfPermMap, Map<Integer, Integer> parentPermMap,
                         Map<Integer, Integer> userPermMap, List<SiteDataPermTree> sites, boolean parentIsTop) {
        //上级不是顶层需要过滤上级的站点id
        if (siteTree != null && !siteTree.isEmpty()) {
            for (AdminSiteTreeVO adminSiteTreeVO : siteTree) {
                boolean need = true;
                if (!parentIsTop) {
                    Set<Integer> parentSiteIdSet = parentPermMap.keySet();
                    //父层没有任何数据则不包含或者完全没有权限的数据（脏数据）
                    if (!parentSiteIdSet.contains(adminSiteTreeVO.getId()) || parentPermMap.get(adminSiteTreeVO.getId()) == 0) {
                        need = false;
                    }
                }
                if (need) {
                    SiteDataPermTree vo = new SiteDataPermTree();
                    vo.setId(adminSiteTreeVO.getId());
                    vo.setParentId(adminSiteTreeVO.getParentId());
                    vo.setName(adminSiteTreeVO.getName());
                    vo.setSite(siteService.findById(vo.getId()));
                    vo.setRowDatas(datas(adminSiteTreeVO.getId(), SITE_TYPE, selfPermMap, parentPermMap, userPermMap, parentIsTop, false));
                    sites.add(vo);
                    convert(adminSiteTreeVO.getChildren(), selfPermMap, parentPermMap, userPermMap, sites, parentIsTop);
                }
            }
        }
    }

    /**
     * 解析
     **/
    private List<CmsDataPermVo.MiniDataUnit> datas(Integer dataId, int dataType, Map<Integer, Integer> selfAuthMap, Map<Integer, Integer> parentPermMap,
                                                   Map<Integer, Integer> userPermMap, boolean parentIsTop, boolean currUserOrOrg) {
        List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
        //当前用户或者当前用户所属组织则不能分配权限，给0 没有取到则认为没权限
        Integer userPermVal = userPermMap.get(dataId) != null ? userPermMap.get(dataId) : 0;
        if (currUserOrOrg) {
            userPermVal = 0;
        }
        int seleteAllVal = SITE_TYPE == dataType ? SITE_ALL_SELECT : (CHANNEL_TYPE == dataType ? CHANNEL_ALL_SELECT : CONTENT_ALL_SELECT);
        int authPermLen = SITE_TYPE == dataType ? SITE_DIGITS : (CHANNEL_TYPE == dataType ? CHANNEL_PERMISSION_DIGITS : CONTENT_DIGITS);
        String su = AuthUtil.toBinary(userPermVal, authPermLen);
        char[] suc = su.toCharArray();
        //解析权限,处理本站点勾选的情况，如果存在，则解析对应的权限勾选情况
        if (selfAuthMap.get(dataId) != null) {
            //得到之前保存的勾选数据
            Integer permVal = selfAuthMap.get(dataId);
            if (parentIsTop) {
                String s = AuthUtil.toBinary(permVal, authPermLen);
                char[] sc = s.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    //原来的操作值从1开始 所以+1
                    //是否可分配是根据用户的权限来
                    dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
                }
                return dataUnits;
            }
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
            Integer parentPermVal = 0;
            //父层都没有配置
            if (parentPermMap.isEmpty()) {
                parentPermVal = selfAuthMap.isEmpty() ? seleteAllVal : 0;
            }
            //父层有配置未包含则是0 没有权限
            if (parentPermMap.get(dataId) != null) {
                parentPermVal = parentPermMap.get(dataId);
            }
            //勾选的数据与限制数据做与运行，得到可以勾选的数据
            String s = AuthUtil.toBinary(permVal & parentPermVal, authPermLen);
            String s1 = AuthUtil.toBinary(parentPermVal, authPermLen);
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                //是否可分配是根据用户的权限来
                //原来的操作值从1开始 所以+1
                dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
            }
        } else {
            //没有取到还有两种情况，一种该站点没有配置，一种完全没有配
            if (selfAuthMap.isEmpty()) {
                //完全没有配置则采用继承
                //如果没勾选的情况下，处理父级为顶层的情况,且自身没有配置权限
                if (parentIsTop && selfAuthMap.isEmpty()) {
                    for (int i = 0; i < authPermLen; i++) {
                        //原来的操作值从1开始 所以+1
                        dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), true));
                    }
                    return dataUnits;
                }
                //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
                // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
                Integer parentPermVal = 0;
                //父层都没有配置
                if (parentPermMap.isEmpty()) {
                    parentPermVal = selfAuthMap.isEmpty() ? seleteAllVal : 0;
                }
                //父层有配置未包含则是0 没有权限
                if (parentPermMap.get(dataId) != null) {
                    parentPermVal = parentPermMap.get(dataId);
                }
                String s1 = AuthUtil.toBinary(parentPermVal, authPermLen);
                char[] scp = s1.toCharArray();
                for (int i = 0; i < scp.length; i++) {
                    //是否可分配是根据用户的权限来
                    //原来的操作值从1开始 所以+1
                    dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(scp[i])));
                }
            } else {
                //该站点没有配置则没有权限
                String s = AuthUtil.toBinary(0, authPermLen);
                char[] sc = s.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    //原来的操作值从1开始 所以+1
                    //是否可分配是根据用户的权限来
                    dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
                }
            }
        }
        return dataUnits;
    }


    private List<CmsDataPermVo.MiniDataUnit> datasByCountPermVal(Integer dataId, int dataType, Integer selfPermVal,
                                                                 Integer userPermVal, boolean isCurrUserOrg) {
        int authPermLen = SITE_TYPE == dataType ? SITE_DIGITS : (CHANNEL_TYPE == dataType ? CHANNEL_PERMISSION_DIGITS : CONTENT_DIGITS);
        List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
        //如果是当前用户或者当前用户所在组织则不可分配权限
        if (isCurrUserOrg) {
            userPermVal = 0;
        }
        String s = AuthUtil.toBinary(selfPermVal, authPermLen);
        String su = AuthUtil.toBinary(userPermVal, authPermLen);
        char[] suc = su.toCharArray();
        char[] sc = s.toCharArray();
        for (int i = 0; i < sc.length; i++) {
            //是否可分配是根据用户的权限来
            //原来的操作值从1开始 所以+1
            dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
        }
        return dataUnits;
    }


    /**
     * 转换数据，整体逻辑如下
     * 1.得到全部的站点数据
     * 2.如果之前勾了就得到勾了的权限，然后解析得到相应的数据
     * 3.判断是否可选，组织则判断上级组织的勾选范围，角色判断所属组织，用户则判断所属组织
     *
     * @param list     栏目数据
     * @param collect  本身勾选的数据
     * @param parent   限制数据
     * @param contents 装载结果集合
     * @param flag     是否特殊处理父级是顶层站点的情况
     */
    private void convertForContent(List<AdminChannelTreeVO> list, Map<Integer, Integer> collect,
                                   Map<Integer, Integer> parent, List<AdminChildContentDataVO> contents,
                                   boolean flag) {
        if (list != null && !list.isEmpty()) {
            for (AdminChannelTreeVO adminChannelTreeVO : list) {
                AdminChildContentDataVO vo = new AdminChildContentDataVO();
                vo.setChannelId(adminChannelTreeVO.getId());
                vo.setParentId(adminChannelTreeVO.getParentId());
                vo.setChannelName(adminChannelTreeVO.getName());
                vo.setDatas(datasForContent(adminChannelTreeVO, collect, parent, flag));
                vo.setFlag(false);
                contents.add(vo);
                convertForContent(adminChannelTreeVO.getChildren(), collect, parent, contents, flag);
            }
        }
    }

    private void convertForChannel(List<AdminChannelTreeVO> list, Map<Integer, Integer> selfPermMap, Map<Integer, Integer> parentPermMap,
                                   Map<Integer, Integer> userPermMap, List<ChannelDataPermTree> channels,
                                   boolean parentIsTop, Integer dataType) {
        if (list != null && !list.isEmpty()) {
            for (AdminChannelTreeVO adminChannelTreeVO : list) {
                boolean need = true;
                if (!parentIsTop) {
                    Set<Integer> parentSiteIdSet = parentPermMap.keySet();
                    //父层没有任何数据则不包含或者完全没有权限的数据（脏数据）
                    if (!parentSiteIdSet.contains(adminChannelTreeVO.getId()) || parentPermMap.get(adminChannelTreeVO.getId()) == 0) {
                        need = false;
                    }
                }
                if (need) {
                    ChannelDataPermTree vo = new ChannelDataPermTree();
                    vo.setId(adminChannelTreeVO.getId());
                    vo.setParentId(adminChannelTreeVO.getParentId());
                    vo.setSiteId(adminChannelTreeVO.getSiteId());
                    vo.setName(adminChannelTreeVO.getName());
                    vo.setRowDatas(datas(adminChannelTreeVO.getId(), dataType, selfPermMap, parentPermMap, userPermMap, parentIsTop, false));
                    channels.add(vo);
                    convertForChannel(adminChannelTreeVO.getChildren(), selfPermMap, parentPermMap, userPermMap, channels, parentIsTop, dataType);
                }
            }
        }
    }

    /**
     * 解析内容权限
     **/
    private Map<String, DataBaseVO> datasForContent(AdminChannelTreeVO adminChannelTreeVO, Map<Integer, Integer> collect,
                                                    Map<Integer, Integer> parent, boolean flag) {
        Map<String, DataBaseVO> map = new HashMap<>(9);
        //解析权限
        if (collect.get(adminChannelTreeVO.getId()) != null) {
            //得到之前保存的勾选数据
            Integer integer = collect.get(adminChannelTreeVO.getId());
            if (flag) {
                String s = AuthUtil.toBinary(integer, CONTENT_DIGITS);
                char[] sc = s.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    map.put("key" + i, new DataBaseVO(i, sc[i], '1'));
                }
                return map;
            }
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            if (parent.get(adminChannelTreeVO.getId()) != null) {
                Integer integerParent = parent.get(adminChannelTreeVO.getId());
                //勾选的数据与限制数据做与运行，得到可以勾选的数据
                String s = AuthUtil.toBinary(integer & integerParent, CONTENT_DIGITS);
                String s1 = AuthUtil.toBinary(integerParent, CONTENT_DIGITS);
                char[] sc = s.toCharArray();
                char[] scp = s1.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    map.put("key" + i, new DataBaseVO(i, sc[i], scp[i]));
                }
            } else {
                //处理上级组织站点删除的情况
                String s = AuthUtil.toBinary(integer, CONTENT_DIGITS);
                char[] sc = s.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    map.put("key" + i, new DataBaseVO(i, sc[i], '0'));
                }
            }
        } else {
            if (flag) {
                for (int i = 0; i < CONTENT_DIGITS; i++) {
                    map.put("key" + i, new DataBaseVO(i, '0', '1'));
                }
                return map;
            }
            //如果组织未勾选
            if (parent.get(adminChannelTreeVO.getId()) != null) {
                Integer integerParent = parent.get(adminChannelTreeVO.getId());
                String s1 = AuthUtil.toBinary(integerParent, CONTENT_DIGITS);
                char[] scp = s1.toCharArray();
                for (int i = 0; i < scp.length; i++) {
                    map.put("key" + i, new DataBaseVO(i, '0', scp[i]));
                }
            } else {
                //处理限制条件站点删除的情况
                for (int i = 0; i < CONTENT_DIGITS; i++) {
                    map.put("key" + i, new DataBaseVO(i, '0', '0'));
                }
            }
        }
        return map;
    }

    /**
     * 解析栏目权限
     */
    private List<CmsDataPermVo.MiniDataUnit> datasForChannel(AdminChannelTreeVO adminChannelTreeVO, Map<Integer, Integer> orgPermMap, Map<Integer, Integer> parentPermMap,
                                                             Map<Integer, Integer> userPermMap, boolean parentIsTop) {
        List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
        Integer userPermVal = userPermMap.get(adminChannelTreeVO.getId());
        String su = AuthUtil.toBinary(userPermVal, SITE_DIGITS);
        char[] suc = su.toCharArray();
        //解析权限,处理本站点勾选的情况，如果存在，则解析对应的权限勾选情况
        if (orgPermMap.get(adminChannelTreeVO.getId()) != null) {
            //得到之前保存的勾选数据
            Integer permVal = orgPermMap.get(adminChannelTreeVO.getId());
            if (parentIsTop) {
                String s = AuthUtil.toBinary(permVal, SITE_DIGITS);
                char[] sc = s.toCharArray();
                for (int i = 0; i < sc.length; i++) {
                    //原来的操作值从1开始 所以+1
                    //是否可分配是根据用户的权限来
                    dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
                }
                return dataUnits;
            }
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
            Integer parentPermVal = 0;
            //父层都没有配置
            if (parentPermMap.isEmpty()) {
                parentPermVal = orgPermMap.isEmpty() ? AuthConstant.SITE_ALL_SELECT : 0;
            }
            //父层有配置未包含则是0 没有权限
            if (parentPermMap.get(adminChannelTreeVO.getId()) != null) {
                parentPermVal = parentPermMap.get(adminChannelTreeVO.getId());
            }
            //勾选的数据与限制数据做与运行，得到可以勾选的数据
            String s = AuthUtil.toBinary(permVal & parentPermVal, SITE_DIGITS);
            String s1 = AuthUtil.toBinary(parentPermVal, SITE_DIGITS);
            char[] sc = s.toCharArray();
            for (int i = 0; i < sc.length; i++) {
                //是否可分配是根据用户的权限来
                //原来的操作值从1开始 所以+1
                dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(sc[i])));
            }
        } else {
            //如果没勾选的情况下，处理父级为顶层的情况,且自身没有配置权限
            if (parentIsTop && orgPermMap.isEmpty()) {
                for (int i = 0; i < SITE_DIGITS; i++) {
                    //原来的操作值从1开始 所以+1
                    dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), true));
                }
                return dataUnits;
            }
            //根据限制条件做判断是否可选，组织对应上级组织，角色对应所属组织，用户对应所属组织
            // 自身未配置则继承上层默认给所有，自身配置了，则默认给没权限，上层配置了以上层为准
            Integer parentPermVal = 0;
            //父层都没有配置
            if (parentPermMap.isEmpty()) {
                parentPermVal = orgPermMap.isEmpty() ? AuthConstant.SITE_ALL_SELECT : 0;
            }
            //父层有配置未包含则是0 没有权限
            if (parentPermMap.get(adminChannelTreeVO.getId()) != null) {
                parentPermVal = parentPermMap.get(adminChannelTreeVO.getId());
            }
            String s1 = AuthUtil.toBinary(parentPermVal, SITE_DIGITS);
            char[] scp = s1.toCharArray();
            for (int i = 0; i < scp.length; i++) {
                //是否可分配是根据用户的权限来
                //原来的操作值从1开始 所以+1
                dataUnits.add(new MiniDataUnit(i + 1, AuthUtil.toBoolean(suc[i]), AuthUtil.toBoolean(scp[i])));
            }
        }
        return dataUnits;
    }

    /**
     * 特殊处理顶级组织以及超管---对应站点权限
     *
     * @param list  站点树数据
     * @param sites 存储数据集合
     */
    private List<SiteDataPermTree> topOrgAndAdminForSite(List<AdminSiteTreeVO> list, List<SiteDataPermTree> sites) {
        if (list != null && !list.isEmpty()) {
            for (AdminSiteTreeVO adminSiteTreeVO : list) {
                SiteDataPermTree vo = new SiteDataPermTree();
                List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
                vo.setId(adminSiteTreeVO.getId());
                vo.setParentId(adminSiteTreeVO.getParentId());
                vo.setName(adminSiteTreeVO.getName());
                vo.setSite(siteService.findById(vo.getId()));
                for (int i = 0; i < SITE_DIGITS; i++) {
                    //原来的操作值从1开始 所以+1
                    dataUnits.add(new CmsDataPermVo.MiniDataUnit(i + 1, false, true));
                }
                vo.setRowDatas(dataUnits);
                sites.add(vo);
                topOrgAndAdminForSite(adminSiteTreeVO.getChildren(), sites);
            }
            sites = SiteDataPermTree.sort(sites);
        }
        return sites;
    }

    private Map<Integer, Integer> topOrgAndAdminForSite(List<AdminSiteTreeVO> list, Map<Integer, Integer> map) {
        if (list != null && !list.isEmpty()) {
            for (AdminSiteTreeVO adminSiteTreeVO : list) {
                map.put(adminSiteTreeVO.getId(), SITE_ALL_SELECT);
                topOrgAndAdminForSite(adminSiteTreeVO.getChildren(), map);
            }
        }
        return map;
    }

    /**
     * 特殊处理顶级组织和超管---对应内容权限
     *
     * @param list     站点树数据
     * @param contents 存储数据集合
     */
    private void topOrgAndAdminForContent(List<AdminChannelTreeVO> list, List<AdminChildContentDataVO> contents) {
        if (list != null && !list.isEmpty()) {
            for (AdminChannelTreeVO adminChannelTreeVO : list) {
                AdminChildContentDataVO vo = new AdminChildContentDataVO();
                Map<String, DataBaseVO> map = new HashMap<>(9);
                vo.setChannelId(adminChannelTreeVO.getId());
                vo.setParentId(adminChannelTreeVO.getParentId());
                vo.setChannelName(adminChannelTreeVO.getName());
                vo.setFlag(false);
                for (int i = 0; i < CONTENT_DIGITS; i++) {
                    map.put("key" + i, new DataBaseVO(i, '1', '1'));
                }
                vo.setDatas(map);
                contents.add(vo);
                topOrgAndAdminForContent(adminChannelTreeVO.getChildren(), contents);
            }
        }
    }

    /**
     * 特殊处理顶级组织和超管---对应栏目权限
     *
     * @param list     站点树数据
     * @param channels 存储数据集合
     * @param dataType
     */
    private void topOrgAndAdminForChannel(List<AdminChannelTreeVO> list, List<ChannelDataPermTree> channels, Integer dataType) {
        if (list != null && !list.isEmpty()) {
            for (AdminChannelTreeVO adminChannelTreeVO : list) {
                ChannelDataPermTree vo = new ChannelDataPermTree();
                List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
                vo.setId(adminChannelTreeVO.getId());
                vo.setParentId(adminChannelTreeVO.getParentId());
                vo.setChannel(channelService.findById(adminChannelTreeVO.getId()));
                vo.setName(adminChannelTreeVO.getName());
                vo.setSiteId(adminChannelTreeVO.getSiteId());
                int channelDigits = CHANNEL_TYPE == dataType ? CHANNEL_PERMISSION_DIGITS : CONTENT_DIGITS;
                for (int i = 0; i < channelDigits; i++) {
                    //原来的操作值从1开始 所以+1
                    dataUnits.add(new CmsDataPermVo.MiniDataUnit(i + 1, false, true));
                }
                vo.setRowDatas(dataUnits);
                channels.add(vo);
                topOrgAndAdminForChannel(adminChannelTreeVO.getChildren(), channels, dataType);
            }
        }
    }

    private Map<Integer, Integer> topOrgAndAdminForChannel(List<AdminChannelTreeVO> list, Map<Integer, Integer> map, int dataType) {
        if (list != null && !list.isEmpty()) {
            int selectAllVal = CHANNEL_TYPE == dataType ? CHANNEL_ALL_SELECT : CONTENT_ALL_SELECT;
            for (AdminChannelTreeVO vo : list) {
                map.put(vo.getId(), selectAllVal);
                topOrgAndAdminForChannel(vo.getChildren(), map, dataType);
            }
        }
        return map;
    }


    /**
     * 得到站点树
     **/
    private String getSiteData() {
        Object o = cacheService.get(SITE_TREE_KEY);
        if (o != null) {
            return o.toString();
        }
        //得到站点树
        return GsonUtils.gsonString(siteService.getSiteTree());
    }

    /**
     * 得到栏目树
     **/
    private String getChannelData(Integer siteId) {
        /**siteId为取所有*/
        if (siteId != null) {
            Object o = cacheService.get(CacheConstant.getChannelKey(siteId));
            if (o != null) {
                return o.toString();
            }
            //得到栏目树
            List<Channel> channels = channelService.getChanelByRecycle(siteId, false);
            channels = channels.stream().sorted(Comparator.comparing(Channel::getSortNum).thenComparing(Comparator.comparing(Channel::getCreateTime))).collect(Collectors.toList());
            List<AdminChannelTreeVO> vos = TreeUtil.convertToTrees(channels, AdminChannelTreeVO::of);
            String data = GsonUtils.gsonString(vos);
            cacheService.set(CacheConstant.getChannelKey(siteId), data);
            return data;
        } else {
            List<CmsSite> sites = siteService.findAll(false, true);
            List<AdminChannelTreeVO> adminChannelTreeVOS = CollUtil.newArrayList();
            for (CmsSite site : sites) {
                String channelData = getChannelData(site.getId());
                if (StringUtils.isNotBlank(channelData)) {
                    adminChannelTreeVOS.addAll(GsonUtils.gsonToList(channelData, AdminChannelTreeVO.class));
                }
            }
            //这里所有栏目的没有加缓存
            String data = GsonUtils.gsonString(adminChannelTreeVOS);
            return data;
        }
    }

    /**
     * 转换树
     **/
    private static List<AdminChildDataVO> convertForTree(List<AdminChildDataVO> child) {
        List<AdminChildDataVO> result = new ArrayList<>();
        Map<Integer, List<AdminChildDataVO>> childrenMap = new HashMap<>(child.size());
        for (AdminChildDataVO t : child) {
            Integer id = t.getSiteId();
            Integer parentId = t.getParentId();
            t.setChildren(new ArrayList<>());
            t.setChildren(childrenMap.computeIfAbsent(id, l -> new ArrayList<>()));
            if (parentId != null) {
                List<AdminChildDataVO> vos = childrenMap.computeIfAbsent(parentId, l -> new ArrayList<>());
                vos.add(t);
            } else {
                result.add(t);
            }
        }
        return result;
    }

    /**
     * 转换树---内容权限
     **/
    private static List<AdminChildContentDataVO> convertForTreeForContent(List<AdminChildContentDataVO> child) {
        List<AdminChildContentDataVO> result = new ArrayList<>();
        Map<Integer, List<AdminChildContentDataVO>> childrenMap = new HashMap<>(child.size());
        for (AdminChildContentDataVO t : child) {
            Integer id = t.getChannelId();
            Integer parentId = t.getParentId();
            t.setChildren(new ArrayList<>());
            t.setChildren(childrenMap.computeIfAbsent(id, l -> new ArrayList<>()));
            if (parentId != null) {
                List<AdminChildContentDataVO> vos = childrenMap.computeIfAbsent(parentId, l -> new ArrayList<>());
                vos.add(t);
            } else {
                result.add(t);
            }
        }
        return result;
    }

    /**
     * 转换树---栏目权限
     **/
    private static List<AdminChildChannelDataVO> convertForTreeForChannel(List<AdminChildChannelDataVO> child) {
        List<AdminChildChannelDataVO> result = new ArrayList<>();
        Map<Integer, List<AdminChildChannelDataVO>> childrenMap = new HashMap<>(child.size());
        for (AdminChildChannelDataVO t : child) {
            Integer id = t.getChannelId();
            Integer parentId = t.getParentId();
            t.setChildren(new ArrayList<>());
            t.setChildren(childrenMap.computeIfAbsent(id, l -> new ArrayList<>()));
            if (parentId != null) {
                List<AdminChildChannelDataVO> vos = childrenMap.computeIfAbsent(parentId, l -> new ArrayList<>());
                vos.add(t);
            } else {
                result.add(t);
            }
        }
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public Map<Integer, Integer> getSiteAuth(String userName) {
        Map<Integer, Integer> map = new HashMap<>(10);
        List<UserAuthBO> userAuthBOS = new ArrayList<>(16);
        CoreUser user = userService.findByUsername(userName);
        if (cacheService.hasKey(CacheConstant.getSiteAuthForUser(user.getOrgId(), user.getId()))) {
            Object o = cacheService.get(CacheConstant.getSiteAuthForUser(user.getOrgId(), user.getId()));
            map = GsonUtil.gsonToMaps(o.toString());
            return map;
        }
        if (user != null) {
            map = getUserAuthMap(user, userAuthBOS, SITE_TYPE, null);
            //保存全部权限数据
            cacheService.set(CacheConstant.getSiteAuthForUser(user.getOrgId(), user.getId()), GsonUtils.gsonString(map));
        }
        return map;
    }

    @Override
    public List<OrgPermVo> getDataPermForOrg(Integer dataId, Integer dataType, Integer siteId, Integer parentId, Boolean getAll) {
        CoreUser user = SystemContextUtils.getCoreUser();
        /**parentId不为空则只取该组织下面的权限*/
        List<OrgPermVo> vos = CollUtil.newArrayList();
        long t1 = System.currentTimeMillis();
        List<CmsOrg> childOrgs = new ArrayList<>();
        if (getAll != null && !getAll) {
            if (parentId == null) {
                childOrgs.add(user.getOrg());
            } else {
                CmsOrg org = orgService.findById(parentId);
                if (org != null) {
                    childOrgs.addAll(org.getChild());
                }
            }
        } else {
            childOrgs.addAll(user.getOrg().getChildNodeList());
        }
        Integer currOrgId = SystemContextUtils.getCoreUser().getOrgId();
        Integer currUserPermVal = authService.countForChannel(siteId, dataId, authService.getUser(SystemContextUtils.getCurrentUsername()), null, null);
        for (CmsOrg org : childOrgs) {
            long t31 = System.currentTimeMillis();
            OrgPermVo orgPermVo = new OrgPermVo();
            orgPermVo.setOrg(org);
            orgPermVo.setId(org.getId());
            orgPermVo.setParentId(org.getParentId());
            //叶子节点，前端需要 存在子组织或者存在所属角色
            if (!org.getChilds().isEmpty() || !org.getRoles().isEmpty()) {
                orgPermVo.setHasChildren(true);
            }
            //取父节点权限
            CmsOrg parentOrg = org.getParent();
            if (org.isTop()) {
                parentOrg = org;
            }

            Integer orgPermVal;
            if (dataType.equals(SITE_TYPE)) {
                orgPermVal = authService.countSiteForOrg(org.getId(), org.getIsTop(), org.getNodeIdList(), dataId);
            } else {
                orgPermVal = authService.countChannelForOrg(org.getId(), org.getIsTop(), org.getNodeIdList(), siteId, dataId, dataType);
            }

            //当前用户或者当前用户所属组织则不能分配权限，给0 没有取到则认为没权限
            boolean isCurrUserOrg = org.getId().equals(currOrgId);
            orgPermVo.setOps(datasByCountPermVal(dataId, dataType, orgPermVal, currUserPermVal, isCurrUserOrg));

            orgPermVo.setChildren(CollUtil.newArrayList());
            //添加角色节点权限
            for (CoreRole role : org.getRoles()) {
                OrgPermVo rolePermVo = new OrgPermVo();
                rolePermVo.setRole(role);
                rolePermVo.setId(org.getId());
                //叶子节点，前端需要 存在子组织或者存在所属角色
                rolePermVo.setHasChildren(false);
                Integer rolePermVal;
                if (dataType.equals(SITE_TYPE)) {
                    rolePermVal = authService.countSiteForRole(role.getOrgid(), role.getOrg().isTop(), role.getId(), role.getOrg().getNodeIdList(), dataId, null);
                } else {
                    rolePermVal = authService.countChannelForRole(role.getOrgid(), role.getOrg().isTop(), role.getId(), role.getOrg().getNodeIdList(), siteId, dataId, dataType, null);
                }
                rolePermVo.setOps(datasByCountPermVal(dataId, dataType, rolePermVal, currUserPermVal, false));
                rolePermVo.setChildren(CollUtil.newArrayList());
                orgPermVo.getChildren().add(rolePermVo);
            }
            vos.add(orgPermVo);
        }
        //构建层级结构
        vos = OrgPermVo.getChildTree(vos);
        return vos;
    }

    private List<CmsDataPermVo.MiniDataUnit> getDataUnitForOrg(Integer dataId, Integer dataType, Integer siteId, CmsOrg org,
                                                               Map<Integer, Integer> parentAuthMap, Map<Integer, Integer> userAuthMap) {
        List<CmsDataPermVo.MiniDataUnit> dataUnits = CollUtil.newArrayList();
        if (org.isTop()) {
            int digits = CHANNEL_TYPE == dataType ? CHANNEL_PERMISSION_DIGITS : (CONTENT_TYPE == dataType ? CONTENT_DIGITS : SITE_DIGITS);
            for (int i = 0; i < digits; i++) {
                //原来的操作值从1开始 所以+1
                dataUnits.add(new CmsDataPermVo.MiniDataUnit(i + 1, false, true));
            }
        } else {
            //得到权限数据
            Map<Integer, Integer> orgAuthMap = getOrgAuthMap(org, true, dataType, siteId);
            //用户有权限还得父层有权限才可分配，此处与运算，userAuthMap 主要用于 convert方法判断是否可分配
            resultWith(userAuthMap, parentAuthMap);
            boolean orgIsCurrUserOrg = org.getId().equals(SystemContextUtils.getCoreUser().getOrgId());
            dataUnits = datas(dataId, dataType, orgAuthMap, parentAuthMap, userAuthMap, false, orgIsCurrUserOrg);
        }
        return dataUnits;
    }

    private List<CmsDataPermVo.MiniDataUnit> getDataUnitForRole(Integer dataId, Integer dataType, Integer siteId, CoreRole role,
                                                                Map<Integer, Integer> parentAuthMap, Map<Integer, Integer> userAuthMap) {
        //得到权限数据
        Map<Integer, Integer> selfAuthMap = getRoleAuthMap(role, CollUtil.newArrayList(), dataType, siteId);
        //用户有权限还得父层有权限才可分配，此处与运算，userAuthMap 主要用于 convert方法判断是否可分配
        resultWith(userAuthMap, parentAuthMap);
        return datas(dataId, dataType, selfAuthMap, parentAuthMap, userAuthMap, false, false);
    }

    @Override
    public Page<UserPermVo> getDataPermForUser(Integer dataId, Integer dataType, Integer siteId, Integer orgid, Integer roleid, String key, Pageable pageable) {
        List<Integer> roleIds = null;
        CmsOrg org = SystemContextUtils.getCoreUser().getOrg();
        List<Integer> orgIds = org.getChildOrgIds();
        if (roleid != null) {
            roleIds = ImmutableList.of(roleid);
        }
        if (orgid != null) {
            orgIds = ImmutableList.of(orgid);
        }
        Page<CoreUser> users = userService.pageUser(true, orgIds, roleIds, key, true,
                CoreUser.AUDIT_USER_STATUS_PASS, null, null, null, null, pageable);
        List<UserPermVo> userPermVos = new ArrayList<>();
        Integer currUserId = SystemContextUtils.getCoreUser().getId();
        Integer currUserPermVal = authService.countForChannel(siteId, dataId, authService.getUser(SystemContextUtils.getCurrentUsername()), null, null);
        for (CoreUser user : users.getContent()) {
            UserPermVo userPermVo = new UserPermVo();
            userPermVo.setUser(user);
            Integer userPermVal;
            if (dataType.equals(SITE_TYPE)) {
                //是否独立配置了站点类权限
                boolean selfAuthSite = countByUserIdAndDataType(user.getId(), SITE_TYPE) > 0;
                userPermVal = authService.countSiteForUser(user.getOrgId(), user.getOrg().getIsTop(), user.getOrg().getNodeIdList(), user.getId(), dataId, selfAuthSite);
            } else {
                userPermVal = authService.countForChannel(siteId, dataId, authService.getUser(user.getUsername()), null, null);
            }
            //当前用户或者当前用户所属组织则不能分配权限，给0 没有取到则认为没权限
            boolean isCurrUserOrg = user.getId().equals(currUserId);
            userPermVo.setOps(datasByCountPermVal(dataId, dataType, userPermVal, currUserPermVal, isCurrUserOrg));
            userPermVos.add(userPermVo);
        }
        Page<UserPermVo> page = new PageImpl<>(userPermVos, pageable, users.getTotalElements());
        return page;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, readOnly = true)
    public Map<Integer, Integer> getChannelAuth(String userName, Integer dataType) {
        Map<Integer, Integer> map = new HashMap<>(10);
        List<UserAuthBO> userAuthBOS = new ArrayList<>(16);
        CoreUser user = userService.findByUsername(userName);
        String cacheKey = CHANNEL_TYPE == dataType ? CacheConstant.getChannelAuthForUser(user.getOrgId(), user.getId()) :
                CacheConstant.getContentAuthForUser(user.getOrgId(), user.getId());
        if (cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            map = GsonUtil.gsonToMaps(o.toString());
            return map;
        }
        if (user != null) {
            //这里是取所有的站点的栏目权限，
            List<CmsSite> sites = siteService.findAll(false, true);
            Map<Integer, Integer> userAuthMap = getUserAuthMapAllSite(user, userAuthBOS, dataType);
            map.putAll(userAuthMap);
            for (CmsSite s : sites) {
                Map<Integer, Integer> userAuthMapBySite = getUserAuthMapBySite(userAuthMap, dataType, s.getId());
                if (CHANNEL_TYPE == dataType) {
                    //保存全部权限数据
                    cacheService.set(CacheConstant.getChannelAuthForUserForSite(user.getOrgId(), user.getId(), s.getId()), GsonUtils.gsonString(userAuthMapBySite));
                }
            }
            if (CHANNEL_TYPE == dataType) {
                //保存全部权限数据
                cacheService.set(CacheConstant.getChannelAuthForUser(user.getOrgId(), user.getId()), GsonUtils.gsonString(map));
            }
            if (CONTENT_TYPE == dataType) {
                cacheService.set(CacheConstant.getContentAuthForUser(user.getOrgId(), user.getId()), GsonUtils.gsonString(map));
            }
        }
        return map;
    }

    @Override
    public Map<Integer, Integer> getSiteChannelAuth(String userName, Integer dataType, Integer siteId) {
        Map<Integer, Integer> map = new HashMap<>(10);
        List<UserAuthBO> userAuthBOS = new ArrayList<>(16);
        CoreUser user = userService.findByUsername(userName);
        String cacheKey = CHANNEL_TYPE == dataType ? CacheConstant.getChannelAuthForUserForSite(user.getOrgId(), user.getId(), siteId) :
                CacheConstant.getContentAuthForUserForSite(user.getOrgId(), user.getId(), siteId);
        if (cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            map = GsonUtil.gsonToMaps(o.toString());
            return map;
        }
        if (user != null) {
            Map<Integer, Integer> userAuthMap = getUserAuthMap(user, userAuthBOS, dataType, siteId);
            Map<Integer, Integer> userAuthMapBySite = getUserAuthMapBySite(userAuthMap, dataType, siteId);
            map.putAll(userAuthMap);
            if (CHANNEL_TYPE == dataType) {
                cacheService.set(CacheConstant.getChannelAuthForUserForSite(user.getOrgId(), user.getId(), siteId), GsonUtils.gsonString(userAuthMapBySite));
            } else {
                cacheService.set(CacheConstant.getContentAuthForUserForSite(user.getOrgId(), user.getId(), siteId), GsonUtils.gsonString(userAuthMapBySite));
            }
        }
        return map;
    }

    @Override
    public AdminUserBO saveUserBo(String username) {
        CoreUser user = userService.findByUsername(username);
        AdminUserBO bo = new AdminUserBO();
        bo.setOrgIds(user.getOrg().getNodeIdList());
        bo.setOrgId(user.getOrgId());
        bo.setUserId(user.getId());
        bo.setTop(user.getOrg().isTop());
        Map<Integer, List<Integer>> site = new HashMap<>(1);
        Map<Integer, List<Integer>> channel = new HashMap<>(10);
        Map<Integer, List<Integer>> content = new HashMap<>(10);
        //用户的权限需要综合角色的权限
        for (CoreRole role : user.getRoles()) {
            site.put(role.getId(), role.getOrg().getNodeIdList());
            channel.put(role.getId(), role.getOrg().getNodeIdList());
            content.put(role.getId(), role.getOrg().getNodeIdList());
        }
        bo.setRoleOrgIdsForSite(site);
        bo.setRoleOrgIdsForChannel(channel);
        bo.setRoleOrgIdsForContent(content);
        //用户是否配置了权限
        int a = countByUserIdAndDataType(user.getId(), AuthConstant.SITE_TYPE);
        bo.setSelfAuthSite(a > 0);
        int b = countByUserIdAndDataType(user.getId(), CHANNEL_TYPE);
        bo.setSelfAuthChannel(b > 0);
        int c = countByUserIdAndDataType(user.getId(), CONTENT_TYPE);
        bo.setSelfAuthContent(c > 0);
        //加入缓存
        cacheService.set(CacheConstant.getAdminUserBO(username), GsonUtils.gsonString(bo));
        return bo;
    }

    @Override
    public void saveOrgAndRoleDataPerm(OrgPermDto dto) throws GlobalException {
        //查询当前站点勾选数据
        List<SysAuthData> authList = getList(null, null, null, null, dto.getDataId(), dto.getDataType().intValue());
        //删除未勾选的
        List<Integer> toDeleteAuthIds = new ArrayList<>();
        List<Integer> toUpdateOrgIds = new ArrayList<>();
        List<SysAuthData> toSaveAuthDatas = new ArrayList<>();
        List<SysAuthData> toUpdateAuthDatas = new ArrayList<>();
        List<Integer> toUpdateRoleIds = new ArrayList<>();
        HashSet<OrgPermUnitDto> perms = dto.getPerms();
        List<OrgPermUnitDto> orgPerms = perms.stream().filter(x -> x.getOrgId() != null).collect(Collectors.toList());
        Integer currSiteId = SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest());
        if (!orgPerms.isEmpty()) {
            Map<Integer, Integer> orgPermMap = orgPerms.stream().collect(Collectors.toMap(OrgPermUnitDto::getOrgId, OrgPermUnitDto::getOrgAuthInt, (key1, key2) -> key2));
            //得到勾选的数据
            Map<Integer, Integer> select = MapUtil.filter(orgPermMap, entry -> {
                if (entry.getValue() > 0) {
                    return true;
                }
                return false;
            });
            //得到取消勾选的数据
            Map<Integer, Integer> noSelect = MapUtil.filter(orgPermMap, entry -> {
                if (entry.getValue() > 0) {
                    return false;
                }
                return true;
            });
            for (Map.Entry<Integer, Integer> entry : select.entrySet()) {
                Integer orgId = entry.getKey();
                Integer value = entry.getValue();
                Optional<SysAuthData> first = authList.stream().filter(o -> orgId.equals(o.getOrgId())).findFirst();
                if (first.isPresent()) {
                    //修改
                    SysAuthData authData = first.get();
                    authData.setDataAuth(value);
                    toUpdateAuthDatas.add(authData);
                } else {
                    //这里不一定是需要新增，有可能继承的,如果有数据则说明单独配置过，则这时候新增，否则不新增如果是继承的权限，这时候新增了反而导致权限变小了
                    List<SysAuthData> orgAuthDataList = dao.getList(Collections.singletonList(orgId), null, null, null, null, dto.getDataType().intValue());
                    if (!orgAuthDataList.isEmpty()) {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(dto.getDataId());
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        authData.setDataAuth(value);
                        authData.setOrgId(orgId);
                        toSaveAuthDatas.add(authData);
                        saveAll(toSaveAuthDatas);
                    } else {
                        //不是都选中则需要处理继承过来的权限数据，全选中则依旧让他采用无数据[继承]
                        boolean siteAllSelect = value < SITE_ALL_SELECT ? false : true;
                        if (CHANNEL_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CHANNEL_ALL_SELECT ? false : true;
                        } else if (CONTENT_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CONTENT_ALL_SELECT ? false : true;
                        }
                        if (!siteAllSelect) {
                            //原来没有配置权限，需要加继承过来的权限
                            CmsOrg org = orgService.findById(orgId);
                            Map<Integer, Integer> orgAuthMapAllSite = getOrgAuthMapAllSite(org, true, dto.getDataType().intValue());
                            orgAuthMapAllSite.forEach((sid, a) -> {
                                SysAuthData authData = new SysAuthData();
                                authData.setDataId(sid);
                                authData.setDataType(dto.getDataType().intValue());
                                authData.setSiteId(currSiteId);
                                if (!sid.equals(dto.getDataId())) {
                                    authData.setDataAuth(a);
                                } else {
                                    authData.setDataAuth(value);
                                }
                                authData.setOrgId(orgId);
                                toSaveAuthDatas.add(authData);
                            });
                            batchSaveForOrg(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                        }
                    }
                }
            }
            noSelect.forEach((dataId, auth) -> {
                List<Integer> authDataList = authList.stream().filter(o -> dataId.equals(o.getOrgId())).map(SysAuthData::getId).collect(Collectors.toList());
                //有数据则删除，无数据则是继承 这个时候 取消选择表示 不想给这条权限，则将继承过来的最大权限都给加上
                if (!authDataList.isEmpty()) {
                    toDeleteAuthIds.addAll(authDataList);
                }
                //未单独分配权限则继承
                if (countByOrgIdAndDataType(dataId, dto.getDataType().intValue()) <= 0) {
                    CmsOrg org = orgService.findById(dataId);
                    Map<Integer, Integer> orgAuthMapAllSite = getOrgAuthMapAllSite(org, true, dto.getDataType().intValue());
                    orgAuthMapAllSite.forEach((sid, a) -> {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(sid);
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        //sid 选项数据值ID 非选中则给0
                        if (!sid.equals(dto.getDataId())) {
                            authData.setDataAuth(a);
                        } else {
                            authData.setDataAuth(0);
                        }
                        authData.setOrgId(dataId);
                        toSaveAuthDatas.add(authData);
                    });
                    batchSaveForOrg(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                }
                toUpdateOrgIds.add(dataId);
            });
            if (!toSaveAuthDatas.isEmpty()) {
                toUpdateOrgIds.addAll(toSaveAuthDatas.stream().filter(x -> x.getOrgId() != null).map(SysAuthData::getOrgId).collect(Collectors.toList()));
            }
            if (!toUpdateAuthDatas.isEmpty()) {
                toUpdateOrgIds.addAll(toUpdateAuthDatas.stream().filter(x -> x.getOrgId() != null).map(SysAuthData::getOrgId).collect(Collectors.toList()));
            }
            if (SITE_TYPE == dto.getDataType()) {
                dealWithCacheForOrgSite(toUpdateOrgIds, dto.getDataId());
            } else if (CHANNEL_TYPE == dto.getDataType()) {
                Channel c = channelService.findById(dto.getDataId());
                if (c != null) {
                    dealWithCacheForOrgChannel(toUpdateOrgIds, c.getSiteId(), dto.getDataId());
                }
            } else if (CONTENT_TYPE == dto.getDataType()) {
                Channel c = channelService.findById(dto.getDataId());
                if (c != null) {
                    dealWithCacheForOrgContent(toUpdateOrgIds, c.getSiteId(), dto.getDataId());
                }
            }

        }
        List<OrgPermUnitDto> rolePerms = perms.stream().filter(x -> x.getRoleId() != null).collect(Collectors.toList());
        if (!rolePerms.isEmpty()) {
            Map<Integer, Integer> rolePermMap = rolePerms.stream().collect(Collectors.toMap(OrgPermUnitDto::getRoleId, OrgPermUnitDto::getRoleAuthInt, (key1, key2) -> key2));
            //得到勾选的数据
            Map<Integer, Integer> select = MapUtil.filter(rolePermMap, entry -> {
                if (entry.getValue() > 0) {
                    return true;
                }
                return false;
            });
            //得到取消勾选的数据
            Map<Integer, Integer> noSelect = MapUtil.filter(rolePermMap, entry -> {
                if (entry.getValue() > 0) {
                    return false;
                }
                return true;
            });
            for (Map.Entry<Integer, Integer> entry : select.entrySet()) {
                Integer roleId = entry.getKey();
                Integer value = entry.getValue();
                Optional<SysAuthData> first = authList.stream().filter(o -> roleId.equals(o.getRoleId())).findFirst();
                if (first.isPresent()) {
                    //修改
                    SysAuthData authData = first.get();
                    authData.setDataAuth(value);
                    toUpdateAuthDatas.add(authData);
                } else {
                    //新增
                    //这里不一定是需要新增，有可能继承的
                    List<SysAuthData> originDataList = dao.getList(null, roleId, null, null, null, dto.getDataType().intValue());
                    //如果有数据则说明单独配置过，则这时候新增
                    if (!originDataList.isEmpty()) {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(dto.getDataId());
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        authData.setDataAuth(value);
                        authData.setRoleId(roleId);
                        toSaveAuthDatas.add(authData);
                        saveAll(toSaveAuthDatas);
                    } else {
                        //不是都选中则需要处理继承过来的权限数据，全选中则依旧让他采用无数据[继承]方式
                        //处理是否全选中
                        boolean siteAllSelect = value < SITE_ALL_SELECT ? false : true;
                        if (CHANNEL_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CHANNEL_ALL_SELECT ? false : true;
                        } else if (CONTENT_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CONTENT_ALL_SELECT ? false : true;
                        }
                        if (!siteAllSelect) {
                            //原来没有配置权限，需要加继承过来的权限
                            CoreRole role = roleService.findById(roleId);
                            Map<Integer, Integer> roleAuthMapAllSite = getRoleAuthMapAllSite(role, CollUtil.newArrayList(), dto.getDataType().intValue());
                            roleAuthMapAllSite.forEach((sid, a) -> {
                                SysAuthData authData = new SysAuthData();
                                authData.setDataId(sid);
                                authData.setDataType(dto.getDataType().intValue());
                                authData.setSiteId(currSiteId);
                                if (!sid.equals(dto.getDataId())) {
                                    authData.setDataAuth(a);
                                } else {
                                    authData.setDataAuth(value);
                                }
                                authData.setRoleId(roleId);
                                toSaveAuthDatas.add(authData);
                            });
                            batchSaveForRole(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                        }
                    }
                }
            }
            noSelect.forEach((dataId, auth) -> {
                List<Integer> authDataList = authList.stream().filter(o -> dataId.equals(o.getRoleId())).map(SysAuthData::getId).collect(Collectors.toList());
                //有数据则删除，无数据则是继承 这个时候 取消选择表示 不想给这条权限，则将继承过来的最大权限都给加上
                if (!authDataList.isEmpty()) {
                    toDeleteAuthIds.addAll(authDataList);
                }
                if (countByRoleIdAndDataType(dataId, dto.getDataType().intValue()) <= 0) {
                    CmsOrg org = roleService.findById(dataId).getOrg();
                    Map<Integer, Integer> orgAuthMapAllSite = getOrgAuthMapAllSite(org, true, dto.getDataType().intValue());
                    orgAuthMapAllSite.forEach((sid, a) -> {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(sid);
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        //sid 选项数据值ID 非选中则给0
                        if (!sid.equals(dto.getDataId())) {
                            authData.setDataAuth(a);
                        } else {
                            authData.setDataAuth(0);
                        }
                        authData.setRoleId(dataId);
                        toSaveAuthDatas.add(authData);
                    });
                    batchSaveForRole(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                }
                toUpdateRoleIds.add(dataId);
            });
        }
        super.physicalDeleteByIdInBatch(toDeleteAuthIds);
        batchUpdate(toUpdateAuthDatas);
        if (!toSaveAuthDatas.isEmpty()) {
            toUpdateRoleIds.addAll(toSaveAuthDatas.stream().filter(x -> x.getRoleId() != null).map(SysAuthData::getRoleId).collect(Collectors.toList()));
        }
        if (!toUpdateAuthDatas.isEmpty()) {
            toUpdateRoleIds.addAll(toUpdateAuthDatas.stream().filter(x -> x.getRoleId() != null).map(SysAuthData::getRoleId).collect(Collectors.toList()));
        }
        if (AuthConstant.SITE_TYPE == dto.getDataType()) {
            dealWithCacheForRolesSite(toUpdateRoleIds, dto.getDataId());
        } else if (CHANNEL_TYPE == dto.getDataType()) {
            Channel c = channelService.findById(dto.getDataId());
            dealWithCacheForChannelRole(toUpdateRoleIds, c.getSiteId(), dto.getDataId());
        } else if (CONTENT_TYPE == dto.getDataType()) {
            Channel c = channelService.findById(dto.getDataId());
            dealWithCacheForRoleContent(toUpdateRoleIds, c.getSiteId(), dto.getDataId());
        }
    }

    @Override
    public void saveUserDataPerm(UserPermDto dto) throws GlobalException {
        //删除未勾选的
        List<Integer> toDeleteAuthIds = new ArrayList<>();
        List<SysAuthData> toSaveAuthDatas = new ArrayList<>();
        List<SysAuthData> toUpdateAuthDatas = new ArrayList<>();
        List<Integer> toUpdateUserIds = new ArrayList<>();
        HashSet<UserPermUnitDto> perms = dto.getPerms();
        Integer currSiteId = SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest());
        List<UserPermUnitDto> userPerms = perms.stream().filter(x -> x.getUserId() != null).collect(Collectors.toList());
        if (!userPerms.isEmpty()) {
            Map<Integer, Integer> userPermMap = userPerms.stream().collect(Collectors.toMap(UserPermUnitDto::getUserId, UserPermUnitDto::getUserAuth, (key1, key2) -> key2));
            //得到勾选的数据
            Map<Integer, Integer> select = MapUtil.filter(userPermMap, entry -> {
                if (entry.getValue() > 0) {
                    return true;
                }
                return false;
            });
            //得到取消勾选的数据
            Map<Integer, Integer> noSelect = MapUtil.filter(userPermMap, entry -> {
                if (entry.getValue() > 0) {
                    return false;
                }
                return true;
            });
            //查询当前站点勾选数据
            List<SysAuthData> authList = getList(null, null, null, null, dto.getDataId(), dto.getDataType().intValue());
            for (Map.Entry<Integer, Integer> entry : select.entrySet()) {
                Integer userId = entry.getKey();
                Integer value = entry.getValue();
                Optional<SysAuthData> first = authList.stream().filter(o -> userId.equals(o.getUserId())).findFirst();
                if (first.isPresent()) {
                    //修改
                    SysAuthData authData = first.get();
                    authData.setDataAuth(value);
                    toUpdateAuthDatas.add(authData);
                } else {
                    //这里不一定是需要新增，有可能继承的,如果有数据则说明单独配置过，则这时候新增，否则不新增如果是继承的权限，这时候新增了反而导致权限变小了
                    List<SysAuthData> orgAuthDataList = dao.getList(null, null, userId, null, null, dto.getDataType().intValue());
                    if (!orgAuthDataList.isEmpty()) {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(dto.getDataId());
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        authData.setDataAuth(value);
                        authData.setUserId(userId);
                        toSaveAuthDatas.add(authData);
                        saveAll(toSaveAuthDatas);
                    } else {
                        //不是都选中则需要处理继承过来的权限数据，全选中则依旧让他采用无数据[继承]
                        boolean siteAllSelect = value < SITE_ALL_SELECT ? false : true;
                        if (CHANNEL_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CHANNEL_ALL_SELECT ? false : true;
                        } else if (CONTENT_TYPE == dto.getDataType().intValue()) {
                            siteAllSelect = value < CONTENT_ALL_SELECT ? false : true;
                        }
                        if (!siteAllSelect) {
                            //原来没有配置权限，需要加继承过来的权限
                            CoreUser user = userService.findById(userId);
                            Map<Integer, Integer> authMapAllSite = getUserAuthMapAllSite(user, CollUtil.newArrayList(), dto.getDataType().intValue());
                            authMapAllSite.forEach((sid, a) -> {
                                SysAuthData authData = new SysAuthData();
                                authData.setDataId(sid);
                                authData.setDataType(dto.getDataType().intValue());
                                authData.setSiteId(currSiteId);
                                if (!sid.equals(dto.getDataId())) {
                                    authData.setDataAuth(a);
                                } else {
                                    authData.setDataAuth(value);
                                }
                                authData.setUserId(userId);
                                toSaveAuthDatas.add(authData);
                            });
                            batchSaveForUser(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                        }
                    }
                }
            }
            noSelect.forEach((dataId, auth) -> {
                List<Integer> authDataList = authList.stream().filter(o -> dataId.equals(o.getUserId())).map(SysAuthData::getId).collect(Collectors.toList());
                //有数据则删除，无数据则是继承 这个时候 取消选择表示 不想给这条权限，则将继承过来的最大权限都给加上
                if (!authDataList.isEmpty()) {
                    toDeleteAuthIds.addAll(authDataList);
                }
                //未单独分配权限则继承
                if (countByUserIdAndDataType(dataId, dto.getDataType().intValue()) <= 0) {
                    CmsOrg org = userService.findById(dataId).getOrg();
                    Map<Integer, Integer> orgAuthMapAllSite = getOrgAuthMapAllSite(org, true, dto.getDataType().intValue());
                    orgAuthMapAllSite.forEach((sid, a) -> {
                        SysAuthData authData = new SysAuthData();
                        authData.setDataId(sid);
                        authData.setDataType(dto.getDataType().intValue());
                        authData.setSiteId(currSiteId);
                        //sid 选项数据值ID 非选中则给0
                        if (!sid.equals(dto.getDataId())) {
                            authData.setDataAuth(a);
                        } else {
                            authData.setDataAuth(0);
                        }
                        authData.setUserId(dataId);
                        toSaveAuthDatas.add(authData);
                    });
                    batchSaveForUser(dto.getDataType(), dto.getDataId(), SystemContextUtils.getSiteId(RequestUtils.getHttpServletRequest()), toSaveAuthDatas);
                }
                toUpdateUserIds.add(dataId);
            });
        }
        super.physicalDeleteByIdInBatch(toDeleteAuthIds);
        batchUpdate(toUpdateAuthDatas);
        if (!toSaveAuthDatas.isEmpty()) {
            toUpdateUserIds.addAll(toSaveAuthDatas.stream().map(SysAuthData::getUserId).collect(Collectors.toList()));
        }
        if (!toUpdateAuthDatas.isEmpty()) {
            toUpdateUserIds.addAll(toUpdateAuthDatas.stream().map(SysAuthData::getUserId).collect(Collectors.toList()));
        }
        if (AuthConstant.SITE_TYPE == dto.getDataType()) {
            dealWithCacheCallForUser(dto.getDataType(), dto.getDataId(), null, toUpdateUserIds);
        } else if (CHANNEL_TYPE == dto.getDataType()) {
            Channel c = channelService.findById(dto.getDataId());
            dealWithCacheCallForUser(dto.getDataType(), dto.getDataId(), c.getSiteId(), toUpdateUserIds);
        } else if (CONTENT_TYPE == dto.getDataType()) {
            Channel c = channelService.findById(dto.getDataId());
            dealWithCacheCallForUser(dto.getDataType(), dto.getDataId(), c.getSiteId(), toUpdateUserIds);
        }
    }


    private void batchSave(Short dataType, Integer dataId, Integer siteId, Iterable<SysAuthData> entities) {
        ThreadUtil.execute(() -> {
            int count = 1;
            Set<SysAuthData> toSaveList = new HashSet<>();
            /**分批删除，否则JPA deleteInBatch会抛出异常*/
            Iterator<SysAuthData> it = entities.iterator();
            try {
                while (it.hasNext()) {
                    toSaveList.add(it.next());
                    count++;
                    if (count % 1000 == 0) {
                        saveAll(toSaveList);
                        //批量保存大量数据库间歇下，数据库容易扛不住
                        ThreadUtil.safeSleep(1000);
                        toSaveList.clear();
                    }
                }
                saveAll(toSaveList);
            } catch (GlobalException e) {
                log.error(e.getMessage());
            }
        });
    }

    private void batchSaveForUser(Short dataType, Integer dataId, Integer siteId, List<SysAuthData> entities) {
        batchSave(dataType, dataId, siteId, entities);
        List<Integer> userIds = entities.stream().map(SysAuthData::getUserId).collect(Collectors.toList());
        dealWithCacheCallForUser(dataType, dataId, siteId, userIds);
    }

    private void batchSaveForRole(Short dataType, Integer dataId, Integer siteId, List<SysAuthData> entities) {
        batchSave(dataType, dataId, siteId, entities);
        List<Integer> userIds = entities.stream().map(SysAuthData::getUserId).collect(Collectors.toList());
        dealWithCacheCallForRole(dataType, dataId, siteId, userIds);
    }

    private void batchSaveForOrg(Short dataType, Integer dataId, Integer siteId, List<SysAuthData> entities) {
        batchSave(dataType, dataId, siteId, entities);
        List<Integer> userIds = entities.stream().map(SysAuthData::getUserId).collect(Collectors.toList());
        dealWithCacheCallForOrg(dataType, dataId, siteId, userIds);
    }

    private void dealWithCacheCallForUser(Short dataType, Integer dataId, Integer siteId, List<Integer> toUpdateUserIds) {
        if (AuthConstant.SITE_TYPE == dataType) {
            dealWithCacheForUserSite(toUpdateUserIds, dataId);
        } else if (CHANNEL_TYPE == dataType) {
            dealWithCacheForChannelUser(toUpdateUserIds, siteId, dataId);
        } else if (CONTENT_TYPE == dataType) {
            dealWithCacheForUserContent(toUpdateUserIds, siteId, dataId);
        }
    }

    private void dealWithCacheCallForRole(Short dataType, Integer dataId, Integer siteId, List<Integer> toUpdateRoleIds) {
        if (AuthConstant.SITE_TYPE == dataType) {
            dealWithCacheForRolesSite(toUpdateRoleIds, dataId);
        } else if (CHANNEL_TYPE == dataType) {
            dealWithCacheForChannelRole(toUpdateRoleIds, siteId, dataId);
        } else if (CONTENT_TYPE == dataType) {
            dealWithCacheForRoleContent(toUpdateRoleIds, siteId, dataId);
        }
    }

    private void dealWithCacheCallForOrg(Short dataType, Integer dataId, Integer siteId, List<Integer> toUpdateOrgIds) {
        if (SITE_TYPE == dataType) {
            dealWithCacheForOrgSite(toUpdateOrgIds, dataId);
        } else if (CHANNEL_TYPE == dataType) {
            dealWithCacheForOrgChannel(toUpdateOrgIds, siteId, dataId);
        } else if (CONTENT_TYPE == dataType) {
            dealWithCacheForOrgContent(toUpdateOrgIds, siteId, dataId);
        }
    }


    @Override
    public void beforeChannelDelete(Integer[] ids) throws GlobalException {
        clearCache();
    }

    @Override
    public void afterChannelSave(Channel c) throws GlobalException {
        List<CoreUser> users = userService.findList(true, null, null, null, true, CoreUser.AUDIT_USER_STATUS_PASS, null, null, null, null,
                maxPaginable);
        for (CoreUser user : users) {
            updateUserChannel(user, c);
        }
        List<CoreRole> roles = roleService.findAll(true);
        for (CoreRole role : roles) {
            updateRoleChannel(role, c);
        }
        List<CmsOrg> orgs = orgService.findAll(true);
        for (CmsOrg org : orgs) {
            updateOrgChannel(org, c);
        }
        clearCache();
    }

    @Override
    public void afterChannelRecycle(List<Channel> channels) throws GlobalException {
        clearCache();
    }

    @Override
    public void afterChannelChange(Channel c) throws GlobalException {
        clearCache();
    }

    @Override
    public void beforeSiteDelete(Integer[] ids) throws GlobalException {
        clearSiteCache();
        clearCache();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void afterSiteSave(CmsSite site) throws GlobalException {
        List<CoreUser> users = userService.findList(true, null, null, null, true, CoreUser.AUDIT_USER_STATUS_PASS, null, null, null, null,
                maxPaginable);
        for (CoreUser user : users) {
            updateUserSite(user, site);
        }
        List<CoreRole> roles = roleService.findAll(true);
        for (CoreRole role : roles) {
            updateRoleSite(role, site);
        }
        List<CmsOrg> orgs = orgService.findAll(true);
        for (CmsOrg org : orgs) {
            updateOrgSite(org, site);
        }
        clearSiteCache();
        clearCache();
    }

    private void clearCache() {
        /** 主动刷新权限缓存 */
        userService.clearAllUserCache();
        orgService.clearAllOrgCache();
        roleService.clearAllRoleCache();
    }

    private void clearSiteCache() {
        if (cacheService.hasKey(SITE_TREE_KEY)) {
            cacheService.del(cacheService.keys(SITE_TREE_KEY));
        }
    }

    private void updateUserChannel(CoreUser user, Channel channel) throws GlobalException {
        /** 栏目类数据权限 */
        /** 是否单独分配了栏目类权限，且有新增栏目类权限 */
        for (int d = CHANNEL_TYPE; d <= CONTENT_TYPE; d++) {
            if (getAssignUserDataPermsByType(user.getId(), d)) {
                SysAuthData newChannelAuth = findAuth(null, null, user.getId(), -channel.getSiteId(), d);
                if (newChannelAuth != null) {
                    SysAuthData authData = findAuth(null, null, user.getId(), channel.getId(), d);
                    if (authData == null) {
                        authData = new SysAuthData();
                    }
                    authData.setDataId(channel.getId());
                    authData.setDataType(d);
                    authData.setSiteId(channel.getSiteId());
                    authData.setDataAuth(newChannelAuth.getDataAuth());
                    authData.setUserId(user.getId());
                    save(authData);
                    super.flush();
                }
            }
        }
    }

    private void updateRoleChannel(CoreRole role, Channel channel) throws GlobalException {
        for (int d = CHANNEL_TYPE; d <= CONTENT_TYPE; d++) {
            if (getAssignRoleDataPermsByType(role.getId(), d)) {
                SysAuthData newChannelAuth = findAuth(null, role.getId(), null, -channel.getSiteId(), d);
                if (newChannelAuth != null) {
                    SysAuthData authData = findAuth(null, role.getId(), null, channel.getId(), d);
                    if (authData == null) {
                        authData = new SysAuthData();
                    }
                    authData.setDataId(channel.getId());
                    authData.setDataType(d);
                    authData.setSiteId(channel.getSiteId());
                    authData.setDataAuth(newChannelAuth.getDataAuth());
                    authData.setRoleId(role.getId());
                    save(authData);
                    super.flush();
                }
            }
        }
    }

    private void updateOrgChannel(CmsOrg org, Channel channel) throws GlobalException {
        /** 文档类数据权限 */
        /** 栏目类数据权限 */
        for (int d = CHANNEL_TYPE; d <= CONTENT_TYPE; d++) {
            if (getAssignOrgDataPermsByType(org.getId(), d)) {
                SysAuthData newChannelAuth = findAuth(org.getId(), null, null, -channel.getSiteId(), d);
                if (newChannelAuth != null) {
                    SysAuthData authData = findAuth(org.getId(), null, null, channel.getId(), d);
                    if (authData == null) {
                        authData = new SysAuthData();
                    }
                    authData.setDataId(channel.getId());
                    authData.setDataType(d);
                    authData.setSiteId(channel.getSiteId());
                    authData.setDataAuth(newChannelAuth.getDataAuth());
                    authData.setOrgId(org.getId());
                    save(authData);
                    super.flush();
                }
            }
        }
    }

    public void updateUserSite(CoreUser user, CmsSite site) throws GlobalException {
        /** 用户是否有新增站点站群权限 */
        if (user.getHasAssignOwnerSite() && user.getOwnerNewSiteOwner()) {
            /** 给用户分配该新建站点的站群权限 */
            Set<CmsSite> sites = user.getOwnerSites();
            sites.add(site);
            user.setSites(sites);
            userService.update(user);
        }
        /** 是否单独分配了站点类权限，且有新增站点类权限 */
        if (getAssignUserDataPermsByType(user.getId(), SITE_TYPE)) {
            //新站点配置的dataId 0
            SysAuthData newChannelAuth = findAuth(null, null, user.getId(), 0, SITE_TYPE);
            if (newChannelAuth != null) {
                SysAuthData authData = findAuth(null, null, user.getId(), site.getId(), SITE_TYPE);
                if (authData == null) {
                    authData = new SysAuthData();
                }
                authData.setDataId(site.getId());
                authData.setDataType(SITE_TYPE);
                authData.setSiteId(site.getId());
                authData.setDataAuth(newChannelAuth.getDataAuth());
                authData.setUserId(user.getId());
                save(authData);
                super.flush();
            }
        }
    }

    private void updateRoleSite(CoreRole role, CmsSite site) throws GlobalException {
        if (role.getHasAssignOwnerSite() && role.getOwnerNewSiteOwner()) {
            Set<CmsSite> sites = role.getOwnerSites();
            CoreRoleAgent.agentOwnerSites(role);
            sites.add(site);
            role.setSites(sites);
            roleService.flush();
        }
        if (getAssignRoleDataPermsByType(role.getId(), SITE_TYPE)) {
            SysAuthData newChannelAuth = findAuth(null, role.getId(), null, 0, SITE_TYPE);
            if (newChannelAuth != null) {
                SysAuthData authData = findAuth(null, role.getId(), null, site.getId(), SITE_TYPE);
                if (authData == null) {
                    authData = new SysAuthData();
                }
                authData.setDataId(site.getId());
                authData.setDataType(SITE_TYPE);
                authData.setSiteId(site.getId());
                authData.setDataAuth(newChannelAuth.getDataAuth());
                authData.setRoleId(role.getId());
                save(authData);
                super.flush();
            }
        }
    }

    private void updateOrgSite(CmsOrg org, CmsSite site) throws GlobalException {
        /** 是否单独分配了站群权限，且有新建站点的站群权限 */
        if (org.getHasAssignOwnerSite() && org.getOwnerNewSiteOwner()) {
            Set<CmsSite> sites = org.getOwnerSites();
            CmsOrgAgent.agentOwnerSites(org);
            sites.add(site);
            org.setSites(sites);
            orgService.flush();
        }
        /** 是否单独分配了站点数据权限，且有新建站点的站点操作数据权限 */
        if (getAssignOrgDataPermsByType(org.getId(), SITE_TYPE)) {
            SysAuthData newChannelAuth = findAuth(org.getId(), null, null, 0, SITE_TYPE);
            if (newChannelAuth != null) {
                SysAuthData authData = findAuth(org.getId(), null, null, site.getId(), SITE_TYPE);
                if (authData == null) {
                    authData = new SysAuthData();
                }
                authData.setDataId(site.getId());
                authData.setDataType(SITE_TYPE);
                authData.setSiteId(site.getId());
                authData.setDataAuth(newChannelAuth.getDataAuth());
                authData.setOrgId(org.getId());
                save(authData);
                //最后一条组织的权限没有更新入库
                super.flush();
            }
        }
    }

    private boolean getAssignUserDataPermsByType(Integer userId, Integer dataType) {
        List<SysAuthData> auths = findAuths(null, null, userId, dataType);
        return !auths.isEmpty();
    }

    private boolean getAssignOrgDataPermsByType(Integer orgId, Integer dataType) {
        List<SysAuthData> auths = findAuths(orgId, null, null, dataType);
        return !auths.isEmpty();
    }

    private boolean getAssignRoleDataPermsByType(Integer roleId, Integer dataType) {
        List<SysAuthData> auths = findAuths(null, roleId, null, dataType);
        return !auths.isEmpty();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
//        ThreadUtil.execute(() -> {
            //预先加载栏目和内容类权限 用户权限
            //这里预先取50个管理员出来先加载 避免在栏目过多的时候 栏目类权限太慢
//            Pageable pageable = PageRequest.of(0, 50);
//            Page<CoreUser> users = userService.pageUser(true, null, null, null, true,
//                    CoreUser.AUDIT_USER_STATUS_PASS, null, null, null, null, pageable);
//            for (CoreUser user : users.getContent()) {
//                getChannelAuth(user.getUsername(), CONTENT_TYPE);
//                getChannelAuth(user.getUsername(), CHANNEL_TYPE);
//                ThreadUtil.safeSleep(30000);
//            }
//        });
    }
}
